from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.utils.device_types import *


class VoiceDeviceControlModule(VoiceModule):

    def __init__(self, priority):
        super(VoiceDeviceControlModule, self).__init__(priority)
        self.devices_map = self.setup_devices_map()

    def find_room(self, text, db):
        with db:
            cur = db.cursor()

            cur.execute("SELECT * FROM ROOMS")

            for room in cur.fetchall():
                position = text.find(room['NAME'].lower())
                if position is not -1:
                    return room
        return None

    def setup_devices_map(self):
        devices_map = {
            WAKE_ON_LAN: ['WAKE_ON_LAN','NAME','DEVICE', WAKE_ON_LAN,'LOCATION'],
            XBOX_ONE_WOL: ['XBOX_ONE_WOL','NAME','ID', XBOX_ONE_WOL,'LOCATION'],
            FUNKSTECKDOSE: ['FUNKSTECKDOSEN', 'NAME', 'DEVICE', FUNKSTECKDOSE, 'ROOM'],
            ZWAVE_SWITCH: ['ZWAVE_SWITCHES', 'NAME', 'ID', ZWAVE_SWITCH, 'LOCATION'],
            URL_SWITCH: ['URL_SWITCH_BINARY', 'NAME', 'ID', URL_SWITCH, 'LOCATION'],
            URL_TOGGLE: ['URL_TOGGLE', 'NAME', 'ID', URL_TOGGLE, 'LOCATION'],
            PHILIPS_HUE_LIGHT: ['PHILIPS_HUE_LIGHTS', 'NAME', 'ID', PHILIPS_HUE_LIGHT, 'LOCATION'],
            URL_RGB_LIGHT: ['URL_RGB_LIGHT', 'NAME', 'ID', URL_RGB_LIGHT, 'LOCATION']
        }

        return devices_map

    def find_devices(self, text, device_types, location_key, db):
        devices = []

        with db:
            cur = db.cursor()

            words = text.split()

            for device_type in device_types:
                item = self.devices_map[device_type]

                if location_key is None:
                    cur.execute("SELECT * FROM " + item[0])
                else:
                    param_array = {'location': location_key}
                    cur.execute("SELECT * FROM " + item[0] + " WHERE " + item[4] + " = :location", param_array)

                for device in cur.fetchall():
                    for word in words:
                        if word == device[item[1]].lower():
                            device_item = {'type': item[3], 'name': device[item[1]], 'id': device[item[2]]}
                            devices.append(device_item)

            '''for item in data:
                if location_key is None:
                    cur.execute("SELECT * FROM "+item[0])
                else:
                    param_array = {'location': location_key}
                    cur.execute("SELECT * FROM "+item[0]+" WHERE "+item[4]+" = :location", param_array)

                for device in cur.fetchall():
                    position = text.find(device[item[1]].lower())
                    if position is not -1:
                        device_item = {'type': item[3], 'name': device[item[1]], 'id': device[item[2]]}
                        devices.append(device_item)'''

        return devices


from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *

def get_permissions(username, db):
    cur = db.cursor()
    cur.execute("SELECT * FROM 'userdata' WHERE USERNAME == :username",
                {'username': username})

    data = cur.fetchone()

    return data

def contains_permission(permissions, permission):
    return ('admin' in permissions or permission in permissions)

def set_permissions(username, permission_user, permissions, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    cur = db.cursor()
    cur.execute("UPDATE 'userdata' SET PERMISSIONS = :permissions WHERE USERNAME == :username",
                {'username':permission_user, 'permissions':permissions})

    return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.DeviceAPI.zwave.utils import do_zwave_request


def get_data(device_id, db):
    return do_zwave_request("/ZAutomation/api/v1/devices/"+device_id, db)

def set_binary_device(device_id, value, db):
    return do_zwave_request("/ZAutomation/api/v1/devices/"+device_id+"/command/"+value, db)

def set_multistate_device(device_id, value, db):
    return do_zwave_request("/ZAutomation/api/v1/devices/"+device_id+"/command/exact?level="+value, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from homevee.cronjobs import IntervalCronjob


class VideoSurveillanceCronjob(IntervalCronjob):
    def __init__(self):
        super(VideoSurveillanceCronjob, self).__init__(task_name="VideoSurveillanceCronjob",
                                                       interval_seconds=15*60)

    def task_to_do(self, *args):
        self.do_video_surveillance()

    def get_seconds_to_wait(self, execution_time=None):
        t = datetime.datetime.today()

        seconds_to_wait = (15 * 60) - ((t.minute * 60) - t.second) % 15 * 60

        return seconds_to_wait

    def do_video_surveillance(self):
        pass


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.Helper.helper_functions import has_permission, get_my_ip
from homevee.utils.database import get_server_data
from homevee.items.Status import *


def generate_key():
    return "123456"

def save_mqtt_device(username, type, location, id, data, db):
    if not has_permission(username, location, db):
        return {'result': 'nopermission'}

    with db:
        cur = db.cursor()

        #if type == MQTT_SENSOR:
        #    cur.execute("INSERT INTO MQTT_SENSORS")
            
        item_data = json.loads(data)
        
        for item in item_data:
            if item['devicetype'] == "sensor":
                cur.execute("INSERT INTO MQTT_SENSORS (NAME, ICON, TYPE, ROOM, SAVE_DATA, DEVICE_ID, VALUE_ID, LAST_VALUE) VALUES (:name, :icon, :type, :room, :save_data, :dev_id, :val_id, \"N/A\")",
                {'name': item['name'], 'icon': item['icon'], 'type': item['sensor_type'],
                'room': location, 'save_data': item['save_data'], 'dev_id': id, 'val_id': item['id']})
        
    return Status(type=STATUS_OK).get_dict()

def generate_device_data(username, location, db):
    if not has_permission(username, location, db):
        return {'result': 'nopermission'}

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM MQTT_DEVICES ORDER BY ID DESC")

        item = cur.fetchone()

        if item is not None:
            new_id = item['ID']+1
        else:
            new_id = 0

        topic = "/home/device/"+str(new_id)

        key = generate_key()

        cur.execute("INSERT INTO MQTT_DEVICES (ID, LOCATION, KEY, TOPIC) VALUES (:id, :location, :key, :topic)",
                    {'id': new_id, 'location': location, 'key': key, 'topic': topic})

        return {'id': new_id, 'topic': topic, 'key': key, 'ip': get_my_ip(),
                'remoteid': get_server_data("REMOTE_ID", db)}

def add_to_intermediates(id, db):
    with db:
        cur = db.cursor()

        cur.execute("INSERT INTO MQTT_DEVICE_INTERMEDIATES (ID) VALUES (:id)",
                    {'id': id})

def is_in_intermediates(id, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT COUNT(*) FROM MQTT_DEVICE_INTERMEDIATES WHERE ID = :id",
                    {'id': id})

        data = cur.fetchone()

        if data['COUNT(*)'] == 0:
            return False
        else:
            return True



#!/usr/bin/python
# -*- coding: utf-8 -*-
import urllib.error
import urllib.parse
import urllib.request

from homevee.DeviceAPI import philips_hue
from homevee.Helper.helper_functions import has_permission
from homevee.utils.device_types import *
from homevee.items.Status import *


def rgb_control(username, devicetype, id, mode, brightness, color, db, check_user=True):
    with db:
        cur = db.cursor()

        if devicetype == URL_RGB_LIGHT:
            cur.execute("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})

            result = cur.fetchone()

            if check_user and not has_permission(username, result['LOCATION'], db):
                return {'result': 'nopermission'}

            output = urllib.request.urlopen("http://"+result['URL']+color).read()

            return Status(type=STATUS_OK).get_dict()
        elif devicetype == PHILIPS_HUE_LIGHT:
            return philips_hue.set_light_mode(username, id, mode, None, brightness, color, db, check_user=True)

def get_rgb_devices(username, room, db):
    if(not has_permission(username, room, db)):
        return {'result': 'nopermission'}

    devices = []

    with db:
        cur = db.cursor()

        #URL_RGB_LIGHT
        cur.execute("SELECT * FROM URL_RGB_LIGHT WHERE LOCATION = :room", {'room': room})
        for item in cur.fetchall():
            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': URL_RGB_LIGHT,
            'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
            devices.append(rgb_item)
        #PHILIPS_HUE
        cur.execute("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE LOCATION = :room AND TYPE = 'color'", {'room': room})
        for item in cur.fetchall():
            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': PHILIPS_HUE_LIGHT,
                        'icon': item['ICON'], 'value': {'is_on': item['IS_ON']== 1, 'brightness': int(item['BRIGHTNESS']/255*100), 'color': item['HUE']}}
            devices.append(rgb_item)

    return {'rgb': devices}

def get_rgb_device(username, type, id, db):
    with db:
        cur = db.cursor()

        #URL_RGB_LIGHT
        if type == URL_RGB_LIGHT:
            cur.execute("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})
            item = cur.fetchone()
            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': URL_RGB_LIGHT,
            'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
            return rgb_item
        #PHILIPS_HUE
        elif type == PHILIPS_HUE_LIGHT:
            cur.execute("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID = :id AND TYPE = 'color'", {'id': id})
            item = cur.fetchone()
            rgb_item = {'name': item['NAME'], 'id': item['ID'], 'type': PHILIPS_HUE_LIGHT,
                        'icon': item['ICON'], 'value': {'is_on': item['IS_ON']== 1,
                        'brightness': int(item['BRIGHTNESS']/255*100), 'color': item['HUE']}}
            return rgb_item


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.request, urllib.error, urllib.parse

from homevee.VoiceAssistant import helper


#todo change route module to class(es)

def get_distance_data(username, text, context, db):

    distance_to_person = get_person_distance_data(username, text, context, db)
    if(distance_to_person is not None):
        return distance_to_person

    distance_to_place = get_person_distance_data(username, text, context, db)
    if(distance_to_place is not None):
        return distance_to_place

    text = text.replace(" an der ", "/")
    text = text.replace("bis nach", "nach")
    words = text.split()

    loc_from = None
    loc_to = None

    is_from = False
    is_to = False
    for i in range(0, len(words)):
        word = words[i]
        if not is_from:
            if word == "von":
                is_from = True
                continue
        else:
            if not is_to:
                if word == "nach":
                    is_to = True
                    continue
                if loc_from is None:
                    loc_from = word
                else:
                    loc_from += ' '+word
            else:
                if loc_to is None:
                    loc_to = word
                else:
                    loc_to += ' '+word

    print((loc_from, loc_to))

    distance, duration = load_distance(loc_from, loc_to)

    minutes = duration/60

    duration_string = str(minutes) + ' Minuten'

    if minutes > 60:
        hours = minutes/60
        minutes = minutes%60

        if hours == 1:
            hour_string = "eine Stunde und"
        else:
            hour_string = str(hours) + " Stunden und"

        if minutes == 1:
            minute_string = "eine Minute"
        else:
            minute_string = str(minutes) + " Minuten"

        duration_string = hour_string + " " + minute_string

    answer_data = [
        ['Von ', loc_from.capitalize(), ' nach ', loc_to.capitalize(), ' sind es ', ['ca', 'etwa', 'in etwa', 'ungefähr'], ' ', distance/1000,
         ' Kilometer und die Fahrt dauert ', ['ca', 'etwa', 'in etwa', 'ungefähr'], ' ', duration_string, '.']
    ]
    output = helper.generate_string(answer_data)

    return {'msg_speech': output, 'msg_text': output}

def get_person_distance_data(username, text, context, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM PEOPLE_DATA")

        person = None

        for item in cur.fetchall():
            if(item['NAME'].lower() in text):
                person = item
            elif item['NICKNAME'].lower in text:
                person = item

        if(person is None):
            return None

    return None

def get_place_distance_data(username, text, context, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM PLACES")

        place = None

        for item in cur.fetchall():
            if (item['NAME'].lower() in text):
                place = item

        if (place is None):
            return None

    return None

def load_distance(loc_from, loc_to):
    url = "http://maps.googleapis.com/maps/api/distancematrix/json?origins="+loc_from+"&destinations="+loc_to+"&mode=driving&language=de-DE&sensor=false"

    result = urllib.request.urlopen(url).read()

    data = json.loads(result)

    distance = data['rows'][0]['elements'][0]['distance']['value']
    duration = data['rows'][0]['elements'][0]['duration']['value']

    return distance, duration


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json

import requests
from dateutil.relativedelta import relativedelta

from homevee.Helper import Logger
from homevee.Helper.helper_functions import load_remote_data
from homevee.items.Status import *


def get_nutrition_data(username, keyword, db):
    data = {'action': 'findsingleitem', 'term': keyword}

    Logger.log("Nutrition item search: "+keyword)

    r = requests.post("https://food.homevee.de/api.php", data=data)
    # Logger.log(r.status_code, r.reason)

    if (r.status_code != 200):
        return None

    response = r.text

    #print response

    data = json.loads(response)

    if(data is None):
        return None

    if('food' not in data):
        return None

    nutrition_data = data['food']

    if(nutrition_data is None):
        return None

    if(len(nutrition_data) is 0):
        return False

    return nutrition_data[0]

def submit_food(username, name, calories, portionsize, portionunit, protein, fat, saturated, unsaturated, carbs, sugar, ean, db):
    remote_data = load_remote_data(username, db)
    remote_id = remote_data['remote_id']

    if(remote_id is None or remote_id == ""):
        return {'result': 'noremoteid'}

    data = {'action': 'submitfood', 'name': name, 'calories': calories, 'portionsize': portionsize, 'portionunit': portionunit,
            'protein': protein, 'fat': fat, 'saturated': saturated, 'unsaturated': unsaturated, 'carbs': carbs,
            'sugar': sugar, 'ean': ean, 'user_remote_id': remote_id}

    r = requests.post("https://food.homevee.de/api.php", data=data)
    #Logger.log(r.status_code, r.reason)

    if(r.status_code != 200):
        return {"result": "error"}

    result = json.loads(r.text)

    if("result" in result):
        if(result['result'] == "ok"):
            return Status(type=STATUS_OK).get_dict()

    return Status(type=STATUS_ERROR).get_dict()

def get_user_day_nutrition_data(username, db):
    with db:
        cur = db.cursor()
    return

def get_user_nutrition_overview(user, db):
    with db:
        cur = db.cursor()

        user_profile = get_user_fitness_profile(user, db)

        Logger.log(user_profile)

        output = {}

        if (user_profile is False):
            has_profile = False
        else:
            has_profile = True

            cur.execute("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                        {'user': user.username, 'date': datetime.datetime.today().strftime("%d.%m.%Y")})

            eaten_calories_today = 0
            eaten_fat_today = 0
            eaten_carbs_today = 0
            eaten_protein_today = 0

            for item in cur.fetchall():

                amount = (float(item['EATEN_PORTION_SIZE'])/float(item['PORTIONSIZE']))

                eaten_calories_today += item['CALORIES']*amount
                eaten_fat_today += item['FAT']*amount
                eaten_carbs_today += item['CARBS']*amount
                eaten_protein_today += item['PROTEIN']*amount

            #print datetime.datetime.today().strftime("%Y-%m-%d")

            nutrition_distribution = {'fat': eaten_fat_today, 'carbs': eaten_carbs_today,
                                      'protein': eaten_protein_today}
            nutrition_day_data = {'calories_goal': int(user_profile['caloriesgoal']), 'calories_left': int(user_profile['caloriesgoal']-eaten_calories_today), 'nutrition_distribution': nutrition_distribution}

            output['nutrition_day_data'] = nutrition_day_data

        weight_progress = {}

        output['has_profile'] = has_profile
        output['weight_progress'] = weight_progress

        return {'data': output}

def save_user_settings(username, height, weight, birthdate, mode, activity, db):
    calories_goal = get_calorie_need(birthdate, weight, height, activity, mode)

    protein_goal, carbs_goal, fat_goal = get_macro_distribution(calories_goal)

    water_goal = 2.0

    add_user_weight(username, weight, db)

    with db:
        cur = db.cursor()

        cur.execute("SELECT COUNT(*) FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': username})

        result = cur.fetchone()['COUNT(*)']

        param_array = {'cals': calories_goal, 'user': username, 'fat': fat_goal, 'saturated': 0, 'carbs': carbs_goal,
                       'sugar': 0, 'protein': protein_goal, 'water': water_goal, 'birthdate': birthdate, 'height': height,
                       'weight': weight, 'activity':activity, 'diet': mode}

        if(result == 0):
            cur.execute("INSERT INTO USER_FITNESS_DATA (USERNAME, CALORIE_GOAL, FAT_GOAL, SATURATED_GOAL, CARBS_GOAL, SUGAR_GOAL, PROTEIN_GOAL, WATER_GOAL, BIRTHDATE, HEIGHT, ACTIVITY_MODE, DIET_MODE) VALUES (:user, :cals, :fat, :saturated, :carbs, :sugar, :protein, :water, :birthdate, :height, :activity, :diet)", param_array)
        else:
            cur.execute("UPDATE USER_FITNESS_DATA SET CALORIE_GOAL = :cals, FAT_GOAL = :fat, SATURATED_GOAL = :saturated, CARBS_GOAL = :carbs, SUGAR_GOAL = :sugar, PROTEIN_GOAL = :protein, WATER_GOAL = :water, BIRTHDATE = :birthdate, HEIGHT = :height, ACTIVITY_MODE = :activity, DIET_MODE = :diet WHERE USERNAME = :user", param_array)

    return {'result': 'ok', 'calories_goal': calories_goal, 'protein_goal': protein_goal, 'carbs_goal': carbs_goal, 'fat_goal': fat_goal}

def get_macro_distribution(calories):

    protein_factor = 0.25
    carbs_factor = 0.5
    fat_factor = 0.25

    protein_goal = (calories/4)*protein_factor
    carbs_goal = (calories/4)*carbs_factor
    fat_goal = (calories/8)*fat_factor

    return int(protein_goal), int(carbs_goal), int(fat_goal)

def load_user_settings(username, db):
    with db:
        cur = db.cursor()

        data = {}

        #{'sitting': 1.2, 'lightactivity': 1.35, 'moderateactivity': 1.55, 'highactivity': 1.75, 'extremeactivity': 1.95}

        data['activities'] = [
            {'title': 'Sitzend', 'description': 'wenig/kein Training + Bürotätigkeit', 'key': 'sitting'},
            {'title': 'leichte Aktivität', 'description': 'geringe tägliche Aktivität + leichtes Training 1-3 Mal/ Woche', 'key': 'lightactivity'},
            {'title': 'moderate Aktivität', 'description': 'moderate tägliche Aktivität + moderates Training 3-5 Mal/ Woche', 'key': 'moderateactivity'},
            {'title': 'hohe Aktivität', 'description': 'körperlich anspruchsvoller Lifestyle + schweres Training/Sport 6-7 Mal/ Woche', 'key': 'highactivity'},
            {'title': 'extrem hohe Aktivität', 'description': 'hartes tägliches Training/Sport und/oder körperlich beanspruchender Job', 'key': 'extremeactivity'}
        ]

        data['modes'] = [
            {'title': 'stark Abnehmen', 'key': 'harddiet'},
            {'title': 'Abnehmen', 'key': 'diet'},
            {'title': 'Gewicht halten', 'key': 'hold'},
            {'title': 'Muskeln aufbauen', 'key': 'gainmuscles'}
        ]

        cur.execute("SELECT * FROM USER_WEIGHT_DATA WHERE USERNAME = :user ORDER BY DATE DESC", {'user': username})
        weight_data = cur.fetchone()['WEIGHT']

        cur.execute("SELECT * FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': username})
        result = cur.fetchone()

        activity_title = None
        for item in data['activities']:
            if(item['key'] == result['ACTIVITY_MODE']):
                activity_title = item['title']
                break

        mode_title = None
        for item in data['modes']:
            if(item['key'] == result['DIET_MODE']):
                mode_title = item['title']
                break

        settings = {'weight': weight_data, 'height': result['HEIGHT'], 'birthdate': result['BIRTHDATE'],'activity': result['ACTIVITY_MODE'],
                    'activity_title': activity_title, 'mode': result['DIET_MODE'], 'mode_title': mode_title}

        data['settings'] = settings

        return {'data': data}

def add_user_weight(username, weight, db):
    with db:
        cur = db.cursor()

        date = datetime.datetime.today().__format__("%Y-%m-%d")

        cur.execute("INSERT INTO USER_WEIGHT_DATA (USERNAME, DATE, WEIGHT) VALUES (:user, :date, :weight)",
                   {'user':username, 'date': date, 'weight': weight})

def delete_food_item(username, id, db):
    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM NUTRITION_DATA WHERE ID = :id AND USER = :user",
                    {'id': id, 'user': username})

        return Status(type=STATUS_OK).get_dict()

def get_calorie_need(birthdate, weight, height, movement_mode, diet_mode):
    # (9.99 x Körpergewicht (in kg)) + (6.25 x Körpergröße (in cm)) – (4.92 x Alter (in Jahren)) – 161

    birthdate = datetime.datetime.strptime(birthdate, "%Y-%m-%d")
    today = datetime.datetime.today()
    age = relativedelta(today, birthdate).years

    base_calories = (9.99 * int(weight)) + (6.25 * int(height)) - (4.92 * age) - 161

    movement_factor = 1.2
    movement_map = {'sitting': 1.2, 'lightactivity': 1.35, 'moderateactivity': 1.55, 'highactivity': 1.75, 'extremeactivity': 1.95}
    if(movement_mode in movement_map):
        movement_factor = movement_map[movement_mode]
    calories_with_movement = base_calories * movement_factor

    diet_factor = 1
    diet_map = {'harddiet': 0.8, 'diet': 0.9, 'hold': 1.0, 'musclegain': 1.1}
    if(diet_mode in diet_map):
        diet_factor = diet_map[diet_mode]
    Logger.log(diet_mode)
    diet_calories = calories_with_movement * diet_factor

    calories_needed = diet_calories

    return int(calories_needed)

def get_user_fitness_profile(user, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM USER_FITNESS_DATA WHERE USERNAME = :user", {'user': user.username})

        result = cur.fetchone()

        if(result is None):
            return False

        data = {'caloriesgoal': result['CALORIE_GOAL'], 'fatgoal': result['FAT_GOAL'], 'saturatedgoal': result['SATURATED_GOAL'],
                'carbsgoal': result['CARBS_GOAL'], 'sugargoal': result['SUGAR_GOAL'], 'proteingoal': result['PROTEIN_GOAL'],
                'watergoal': result['WATER_GOAL']}

        return data

def get_user_weight_progress(username, start, end, db):
    start = datetime.datetime.strptime(start, "%d.%m.%Y").strftime("%Y-%m-%d")
    end = datetime.datetime.strptime(end, "%d.%m.%Y").strftime("%Y-%m-%d")

    with db:
        cur = db.cursor()

        weight_items = []

        cur.execute("SELECT WEIGHT, strftime(\"%d.%m.%Y\", DATE) as DATETIME FROM USER_WEIGHT_DATA WHERE USERNAME = :user AND DATE >= :start AND DATE <= :ende ORDER BY DATE ASC",
                    {'user': username, 'start': start, 'end': end})

        for item in cur.fetchall():
            weight_items.append({'date': item['DATETIME'], 'weight': item['WEIGHT']})

        return {'progress': weight_items}

def get_user_day_nutrition_items(username, date, db):
    with db:
        cur = db.cursor()

        daytime_items = {'morning': [], "noon": [], "evening": [], "snacks": []}

        cur.execute("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                    {'user': username, 'date': date})

        for item in cur.fetchall():
            food_item = {'id': item['ID'], 'daytime': item['DAYTIME'], 'name': item['FOOD_NAME'], 'eatenportionsize': item['EATEN_PORTION_SIZE'],
                         'portionsize': item['PORTIONSIZE'], 'portionunit': item['PORTIONUNIT'], 'calories': item['CALORIES'],
                         'fat': item['FAT'], 'saturated': item['SATURATED'], 'unsaturated': item['UNSATURATED'],
                         'carbs': item['CARBS'], 'sugar': item['SUGAR'], 'protein': item['PROTEIN'], 'date': item['DATE']}

            if(item['DAYTIME'] in daytime_items):
                daytime_items[item['DAYTIME']].append(food_item)
            else:
                daytime_items['snacks'].append(food_item)

        calories_goal = get_user_fitness_profile(username, db)['caloriesgoal']

        return {'items': daytime_items, 'caloriesgoal': calories_goal}

def move_item(username, id, date, daytime, deleteold, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM NUTRITION_DATA WHERE ID = :id", {'id': id})
        item = cur.fetchone()

        if(item is None):
            return {'result': 'itemnotexisting'}

        name = item['FOOD_NAME']
        eatenportionsize = item['EATEN_PORTION_SIZE']
        portionsize = item['PORTIONSIZE']
        portionunit = item['PORTIONUNIT']
        calories = item['CALORIES']
        fat = item['FAT']
        saturated = item['SATURATED']
        unsaturated = item['UNSATURATED']
        carbs = item['CARBS']
        sugar = item['SUGAR']
        protein = item['PROTEIN']

        add_edit_user_day_nutrition_item(username, None, date, daytime, name, eatenportionsize, portionsize, portionunit,
                                         calories, fat, saturated, unsaturated, carbs, sugar, protein, db)

        if deleteold == "true" or deleteold == True or deleteold == 1 or deleteold == "1":
            cur.execute("DELETE FROM NUTRITION_DATA WHERE ID = :id", {'id': id})

    return Status(type=STATUS_OK).get_dict()

def add_edit_user_day_nutrition_item(username, id, date, daytime, name, eatenportionsize, portionsize, portionunit, calories, fat, saturated, unsaturated, carbs, sugar, protein, db):
    add_new = (id == None or id == "" or id == "-1")

    with db:
        cur = db.cursor()

        param_array = {'user': username, 'date': date, 'daytime': daytime, 'name': name, 'eatenportionsize': eatenportionsize,
                       'portionsize': portionsize, 'portionunit': portionunit, 'calories': calories, 'fat': fat,
                       'saturated': saturated, 'unsaturated': unsaturated, 'carbs': carbs, 'sugar': sugar, 'protein': protein}

        if add_new:
            cur.execute("INSERT INTO NUTRITION_DATA (USER, DATE, DAYTIME, FOOD_NAME, EATEN_PORTION_SIZE, PORTIONSIZE, PORTIONUNIT, CALORIES, FAT, SATURATED, UNSATURATED, CARBS, SUGAR, PROTEIN) VALUES (:user, :date, :daytime, :name, :eatenportionsize, :portionsize, :portionunit, :calories, :fat, :saturated, :unsaturated, :carbs, :sugar, :protein)",
                        param_array)
        else:
            param_array['id'] = id

            cur.execute("UPDATE NUTRITION_DATA SET USER = :user, DATE = :date, DAYTIME = :daytime, FOOD_NAME = :name, EATEN_PORTION_SIZE = :eatenportionsize, PORTIONSIZE = :portionsize, PORTIONUNIT = :portionunit, CALORIES = :calories, FAT = :fat, SATURATED = :saturated, UNSATURATED = :unsaturated, CARBS = :carbs, SUGAR = :sugar, PROTEIN = :protein WHERE ID = :id",
                        param_array)

    return Status(type=STATUS_OK).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

import requests

from homevee.items.Gateway import Gateway


def do_zwave_request(path, db):
    try:
        gateway = Gateway.load_from_db("Z-Wave", db)
        port = ""
        if (gateway.port != None and gateway.port != ""):
            port = ":" + str(gateway.port)

        url = "http://" + gateway.key1 + ':' + gateway.key2 + '@' + gateway.ip + port + path

        #print url

        headers = {'content-type': 'application/json', 'Accept-Charset': 'UTF-8'}
        request = requests.get(url, headers=headers)
        data = request.content

        #print data

        json_data = json.loads(data)

        #print url, json_data

        return json_data
    except:
        return None


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib.error
import urllib.parse
import urllib.request
from homevee.items.Status import *
from homevee.Helper import Logger
from homevee.items.Gateway import *


def control_blinds(id, goto, db):
    gateway = Gateway.load_from_db(RADEMACHER_HOMEPILOT, db)

    url = "http://"+gateway.ip+"/deviceajax.do?cid=9&did="+str(id)+"&goto="+str(goto)+"&command=0"

    Logger.log(url)

    response = urllib.request.urlopen(url).read()

    data = json.loads(response)

    if(data['status'] != 'uisuccess'):
        return Status(type=STATUS_ERROR).get_dict()

    Logger.log(response)

    with db:
        cur = db.cursor()

        cur.execute("UPDATE HOMEPILOT_BLIND_CONTROL SET LAST_POS = :pos WHERE ID == :id",
                    {'pos': goto, 'id': id})
    return Status(type=STATUS_OK).get_dict()


import os

from homevee.DeviceAPI import xbox_wol
from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *


def wake_on_lan(username, id, db, check_user=True):

    cur = db.cursor()
    cur.execute("SELECT * FROM WAKE_ON_LAN WHERE DEVICE == :id", {'id': id})

    device = cur.fetchone()

    if check_user and not has_permission(username, device['LOCATION'], db):
        return {'result': 'nopermission'}

    os.system("sudo wakeonlan "+device['MAC_ADDRESS'])

    cur.close()

    return Status(type=STATUS_OK).get_dict()

def get_wol_devices(username, room, db):
    if not has_permission(username, room, db):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    devices = []

    cur = db.cursor()
    cur.execute("SELECT * FROM WAKE_ON_LAN WHERE LOCATION == :room", {'room': room})

    for data in cur.fetchall():
        devices.append({'name': data['NAME'], 'id': data['DEVICE'], 'icon': data['ICON']})

    return {'devices': devices}

def wake_xbox_on_lan(username, id, db, check_user=True):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM XBOX_ONE_WOL WHERE ID == :id",
                    {'id': id})

        device = cur.fetchone()

        if check_user and not has_permission(username, device['LOCATION'], db):
            return {'result': 'nopermission'}

        cur.close()

        ip = device['IP_ADDRESS']
        live_id = device['XBOX_LIVE_ID']

        xbox_wol.xbox_wake_up(ip, live_id)

        return Status(type=STATUS_OK).get_dict()

def get_xbox_devices(username, room, db):
    if not has_permission(username, room, db):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    devices = []

    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM XBOX_ONE_WOL WHERE LOCATION == :room", {'room': room})

        for data in cur.fetchall():
            devices.append({'name': data['NAME'], 'id': data['ID'], 'icon': data['ICON']})

        cur.close()

        return {'devices': devices}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.Helper.helper_functions import has_permission
from homevee.items.Status import *


def add_edit_heating_scheme_item(username, id, time, value, active, days, data, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    with db:
        cur = db.cursor()

        params = {'time': time, 'value': value, 'active': active}

        if id == "" or id is None:
            cur.execute("INSERT INTO HEATING_SCHEME (TIME, VALUE, ACTIVE) VALUES (:time, :value, :active)",
                        params)

            id = cur.lastrowid
        else:
            params['id'] = id
            cur.execute("UPDATE HEATING_SCHEME SET TIME = :time, VALUE = :value, ACTIVE = :active WHERE ID = :id",
                params)

        #HEATING_SCHEME_DAYS bearbeiten
        cur.execute("DELETE FROM 'HEATING_SCHEME_DAYS' WHERE HEATING_SCHEME_ID = :id", {'id': id})

        day_array = json.loads(days)

        for day in day_array:
            cur.execute("INSERT INTO 'HEATING_SCHEME_DAYS' (HEATING_SCHEME_ID, WEEKDAY_ID) VALUES (:id, :weekday_id)",
                {'id': id, 'weekday_id': day})

        #HEATING_SCHEME_DEVICES bearbeiten
        cur.execute("DELETE FROM 'HEATING_SCHEME_DEVICES' WHERE ID = :id", {'id': id})

        device_array = json.loads(data)

        for device in device_array['devices']:
            cur.execute("INSERT INTO 'HEATING_SCHEME_DEVICES' (ID, LOCATION, TYPE, DEVICE_ID) VALUES (:id, :location, :type, :device_id)",
                {'id': id, 'location': device['location'], 'type': device['type'], 'device_id': device['id']})

        return Status(type=STATUS_OK).get_dict()

def delete_heating_scheme_item(username, id, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    params = {'id': id}

    queries = [
        "DELETE FROM HEATING_SCHEME WHERE ID = :id",
        "DELETE FROM HEATING_SCHEME_DAYS WHERE HEATING_SCHEME_ID = :id",
        "DELETE FROM HEATING_SCHEME_DEVICES WHERE ID = :id"
    ]

    with db:
        cur = db.cursor()

        for query in queries:
            cur.execute(query, params)

    cur.close()

    return Status(type=STATUS_OK).get_dict()

def get_heating_scheme_items(username, day, rooms, db):
    if not has_permission(username, "admin", db):
        return "noadmin"

    params = {'day': day}
    query_in_rooms_string = ""

    if rooms is not None and rooms != "":
        params['rooms'] = rooms
        query_in_rooms_string = "AND LOCATION IN (:rooms)"

    with db:
        cur = db.cursor()
        cur.execute("SELECT HEATING_SCHEME.ID, TIME, VALUE, ACTIVE, WEEKDAY_ID, ROOMS.NAME as LOCATION, TYPE, DEVICE_ID FROM HEATING_SCHEME, HEATING_SCHEME_DAYS, HEATING_SCHEME_DEVICES, ROOMS WHERE HEATING_SCHEME.ID = HEATING_SCHEME_DAYS.HEATING_SCHEME_ID AND HEATING_SCHEME.ID = HEATING_SCHEME_DEVICES.ID AND HEATING_SCHEME_DEVICES.LOCATION = ROOMS.LOCATION AND WEEKDAY_ID = :day "+query_in_rooms_string+" ORDER BY TIME",
                    params)

        heating_scheme_items = {}
        heating_scheme_data = {}

        results = cur.fetchall()

        for result in results:
            if result['ID'] not in heating_scheme_data:
                heating_scheme_data[result['ID']] = []

            heating_scheme_data[result['ID']].append({'location': result['LOCATION'], 'type': result['TYPE'], 'device':result['DEVICE_ID']})

            heating_scheme_items[result['ID']] = {'time': result['TIME'], 'value': result['VALUE'], 'isactive': (True if result['ACTIVE']=="true" else False)}

        for result in results:
            heating_scheme_items[result['ID']]['data'] = heating_scheme_data[result['ID']]

        cur.close()

        return {'heatingscheme': heating_scheme_items}

def get_heating_scheme_item_data(username, id, db):
    heating_scheme_item = {}

    params = {'id': id}


    with db:
        #Tage abfragen
        chosen_days = []
        cur = db.cursor()
        cur.execute("SELECT WEEKDAY_ID FROM HEATING_SCHEME_DAYS WHERE HEATING_SCHEME_ID == :id", params)
        for day in cur.fetchall():
            chosen_days.append(int(day['WEEKDAY_ID']))
        heating_scheme_item['days'] = chosen_days

        #Geräte abfragen
        devices = []
        cur.execute("SELECT * FROM HEATING_SCHEME_DEVICES WHERE ID == :id", params)
        for device in cur.fetchall():
            devices.append({'id': device['DEVICE_ID'], 'type': device['TYPE'], 'location': device['LOCATION']})
        heating_scheme_item['devicearray'] = json.dumps({'devices': devices})

        #Daten
        cur.execute("SELECT * FROM HEATING_SCHEME WHERE ID == :id", params)
        scheme_item_data = cur.fetchone()

        heating_scheme_item['value'] = float(scheme_item_data['VALUE'])
        heating_scheme_item['time'] = scheme_item_data['TIME']
        heating_scheme_item['active'] = scheme_item_data['ACTIVE']

        cur.close()

        return {'heatingschemeitem': heating_scheme_item}

def set_heating_scheme_item_active(username, id, active, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    with db:
        cur = db.cursor()
        cur.execute("UPDATE HEATING_SCHEME SET ACTIVE = :active WHERE ID = :id",
            {'active': active, 'id': id})

        cur.close()

        #Abfrage erfolgreich?
        if True:
            return Status(type=STATUS_OK).get_dict()
        else:
            return Status(type=STATUS_ERROR).get_dict()

def is_heating_scheme_active(username, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    active = False

    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM SERVER_DATA WHERE KEY == 'HEATING_SCHEME_ACTIVE'")
        value = cur.fetchone()['VALUE']

        if value == "true":
            active = True

        cur.close()

        return {'isactive': active}

def set_heating_scheme_active(username, active, db):
    if not has_permission(username, "admin", db):
        return Status(type=STATUS_NO_ADMIN).get_dict()

    with db:
        cur = db.cursor()
        cur.execute("INSERT OR REPLACE INTO SERVER_DATA (KEY, VALUE) VALUES (:key, :value)",
            {'key': "HEATING_SCHEME_ACTIVE", 'value': active})

        cur.close()

        #Abfrage erfolgreich?
        if True:
            return Status(type=STATUS_OK).get_dict()
        else:
            return Status(type=STATUS_ERROR).get_dict()


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee import Helper, Updater
from homevee.DeviceAPI.set_modes import set_modes
from homevee.Functions import ar_control, people_classifier
from homevee.Helper import translations
from homevee.utils.database import get_server_data
from .DeviceAPI import dimmer, energy_data
from .DeviceAPI.blind_control import set_blinds, get_all_blinds, set_room_blinds
from .DeviceAPI.get_modes import *
from .DeviceAPI.heating import get_thermostats, control_room_heating, heating_control
from .DeviceAPI.rfid_control import get_rfid_tags, add_edit_rfid_tag, run_rfid_action
from .DeviceAPI.rgb_control import rgb_control
from .DeviceAPI.wake_on_lan import *
# from Functions import people_classifier, ar_control
# from Functions.people_classifier import *
from .Functions import room_data, weather, events, chat, shopping_list, media_center, gps_data, nutrition_data
from .Functions.calendar import *
from .Functions.condition_actions.actions import run_scene
from .Functions.graph_data import *
from .Functions.home_budget import get_home_budget_data, add_edit_home_budget_data, delete_home_budget_data, \
    get_home_budget_data_day_items, get_home_budget_data_graph
from .Functions.sensor_data import *
from .Functions.system_info import *
from .Functions.tv_data import *
from .Helper.helper_functions import load_remote_data, save_remote_data, set_remote_control_enabled, \
    connect_remote_id_with_account, verify_user
from .Manager import dashboard, mqtt_connector, places
from .Manager.api_key import *
from .Manager.automation import *
from .Manager.custom_voice_commands import get_voice_commands, add_edit_voice_command, delete_voice_command
from .Manager.device import *
from .Manager.gateway import *
from .Manager.heating_scheme import *
from .Manager.persons import get_persons, add_edit_person
from .Manager.room import *
from .Manager.scenes import add_edit_scene, delete_scene, get_all_scenes
from .Manager.smart_speaker import get_smart_speakers
from .Manager.user import *
from .VoiceAssistant import *
from .VoiceAssistant.voice_replace_manager import *


def process_data(data, db):
    try:
        username = data['username']
        password = data['password']

        if 'language' in data and data['language'] is not None:
            language = data['language']
        else:
            language = translations.LANGUAGE

        verified = verify_user(username, password)

        if not verified:
            return json.dumps({'result': 'wrongdata'})

        try:
            return handle_request(data, db)
        except Exception as e:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            return
    except Exception as e:
        if(Logger.IS_DEBUG):
                traceback.print_exc()
        if 'status' in data:
            if data['status'] == 'connectionok':
                Logger.log('connectionok')


def handle_request(request, db):
    response = None

    action = request['action']
    username = request['username']

    user = User.load_username_from_db(username, db)

    if 'language' in request:
        language = request['language']
    else:
        language = translations.LANGUAGE

    if ('user_last_location' in request and request['user_last_location'] is not None):
        user_last_location = json.loads(request['user_last_location'])
    else:
        user_last_location = None

    # Roomdata
    if action == "login":
        rooms = room_data.get_rooms(user, db)

        remote_id = get_server_data("REMOTE_ID", db)

        response = {'remote_id': remote_id, 'rooms': rooms['rooms']}

    # Roomdata
    elif action == "getrooms":
        response = room_data.get_rooms(user, db)
    elif action == "getroomdata":
        response = room_data.get_room_data(user, request['room'], db)

    # Remote-ID
    elif action == "setremotedata":
        response = save_remote_data(user, request['remote_id'], request['linked_account'], db)
    elif action == "getremotedata":
        response = load_remote_data(user, db)
    elif action == "setremotecontrolenabled":
        response = set_remote_control_enabled(user, request['enabled'], db)
    elif action == "connectremoteidwithaccount":
        response = connect_remote_id_with_account(user, request['accountname'], request['accountsecret'], db)

    # Kalender
    elif action == "getcalendaritemdates":
        response = get_calendar_item_dates(user, request['year'], db)
    elif action == "getcalendardayitems":
        response = get_calendar_day_items(user, request['date'], db)
    elif action == "addeditcalendarentry":
        response = add_edit_entry(user, request['id'], request['name'], request['date'], request['start'],
                                  request['end'], request['note'], request['address'], db)
    elif action == "deletecalendarentry":
        response = delete_entry(user, request['id'], db)

    # Personen
    elif action == "getpersons":
        response = get_persons(db)
    elif action == "addeditperson":
        response = add_edit_person(user, request['id'], request['name'], request['nickname'], request['address'],
                                   request['latitude'], request['longitude'], request['phonenumber'],
                                   request['birthdate'], db)

    # RGB Lampen
    elif action == "setrgb":
        response = rgb_control(user, request['type'], request['id'], request['mode'], request['brightness'],
                               request['color'], db)

    # RFID
    elif action == "getrfidtags":
        response = get_rfid_tags(user, db)
    elif action == "addeditrfidtag":
        response = add_edit_rfid_tag(user, request['name'], request['uuid'], request['type'], request['id'], db)
    elif action == "deleterfidtag":
        response = add_edit_rfid_tag(user, request['uuid'], db)
    elif action == "runrfidaction":
        response = run_rfid_action(user, request['uuid'], db)

    # Events
    elif action == "getevents":
        type = None
        if "type" in request:
            type = request['type']
        response = events.get_events(user, type, request['limit'], request['offset'], db)
    elif action == "geteventtypes":
        response = events.get_event_types(db)
    elif action == "addevent":
        response = events.add_event(request['type'], request['text'], db)
    elif action == "getunseenevents":
        response = events.get_unseen_events(user, db)

    # Permissions
    elif action == "getpermissions":
        response = Helper.permissions.get_permissions(request['user'], db)
    elif action == "setpermissions":
        response = Helper.permissions.set_permissions(user, request['user'], request['permissions'], db)

    # MQTT Anlernen => generate_device_data()
    elif action == "generatedevicedata":
        response = mqtt_connector.generate_device_data(user, request['location'], db)
    elif action == "savemqttdevice":
        response = mqtt_connector.save_mqtt_device(user, request['type'], request['location'], request['id'],
                                                   request['devicedata'], db)

    # Jalousie-Steuerung
    elif action == "setblinds":
        response = set_blinds(user, request['type'], request['id'], request['value'], db)
    elif action == "setroomblinds":
        response = set_room_blinds(user, request['location'], request['value'], db)
    elif action == "getallblinds":
        response = get_all_blinds(user, db)

    # Weather
    elif action == "getweather":
        response = weather.get_weather(int(request['daycount']), db)
    elif action == "getweathercitylist":
        response = weather.get_weather_city_list(db)
    elif action == "setweathercityid":
        response = weather.set_weather_city_id(user, request['id'])

    # Get graph data
    elif action == "getgraphdata":
        response = get_graph_data(user, request['room'], request['type'], request['id'], request['von'], request['bis'],
                                  db)

    # Get modes, set modes
    elif action == "getmodes":
        response = get_modes(user, request['room'], request['type'], request['device'], db)
    elif action == "setmodes":
        response = set_modes(user, request['type'], request['device'], request['zustand'], db)

    # Dimmer
    elif action == "setdimmer":
        response = dimmer.set_dimmer(user, request['type'], request['id'], request['value'], db)

    # Wake on Lan
    elif action == "wakeonlan":
        response = wake_on_lan(user, request['id'], db)
    elif action == "startxboxone":
        response = wake_xbox_on_lan(user, request['id'], db)

    # Systeminfo
    elif action == "getsysteminfo":
        response = get_system_info()

    # Chat
    elif action == "getchatmessages":
        response = chat.get_chat_messages(user, request['time'], request['limit'], db)
    elif action == "getchatimage":
        response = chat.get_chat_image(user, request['imageid'], db)
    elif action == "sendchatmessage":
        response = chat.send_chat_message(user, request['data'], db)

    # Dashboard
    elif action == "getuserdashboarditems":
        response = dashboard.get_user_dashboard_items(user, db)
    elif action == "getuserfavouritedevices":
        response = dashboard.get_user_favourite_devices(user, db)
    elif action == "edituserdashboard":
        response = dashboard.edit_user_dashboard(user, request['dashboarddata'], db)
    elif action == "getuserdashboard":
        response = dashboard.get_user_dashboard(user, db)

    # Energie-Daten
    elif action == "getenergydata":
        response = energy_data.get_energy_data(user, request['room'], request['devicetype'], request['deviceid'],
                                               request['von'], request['bis'], db)
    elif action == "getdeviceenergydata":
        response = energy_data.get_device_energy_data(user, request['room'], request['devicetype'], request['deviceid'],
                                                      request['von'], request['bis'], db)
    elif action == "getenergycourse":
        response = energy_data.get_energy_course(user, request['room'], request['von'], request['bis'], db)
    elif action == "setpowercost":
        response = energy_data.set_power_cost(user, request['cost'], db)
    elif action == "getpowercost":
        response = energy_data.get_power_cost(user, db)

    # Sensor-Daten
    elif action == "getsensordata":
        type = None
        if 'type' in request: type = request['type']

        id = None
        if 'id' in request: id = request['id']

        showeinheit = None
        if 'showeinheit' in request: showeinheit = request['showeinheit']
        response = get_sensor_data(user, request['room'], type, id, showeinheit, db)

    # User-Manager
    elif action == "getusers":
        response = get_users(user, db)
    elif action == "deleteuser":
        response = delete_user(user, request['name'], db)
    elif action == "addedituser":
        response = add_edit_user(user, request['name'], request['psw'], request['ip'], request['permissions'], db)

    # Gateway-Manager
    elif action == "getgateways":
        response = get_gateways(user, db)
    elif action == "addeditgateway":
        response = add_edit_gateway(user, request['type'], request['usr'], request['psw'], request['changepw'],
                                    request['ip'], request['port'], request['gateway_type'], db)
    elif action == "deletegateway":
        response = delete_gateway(user, request['type'], db)
    elif action == "getgatewaydevices":
        response = get_gateway_devices(user, request['type'], db)
    elif action == "connectgateway":
        response = connect_gateway(user, request['type'], request['ip'], db)

    # TV-Programm
    elif action == "settvchannels":
        response = set_tv_channels(user, request['channels'], db)
    elif action == "gettvplan":
        response = get_tv_plan(user, request['time'], db)
    elif action == "getalltvchannels":
        response = get_all_tv_channels(user, db)

    # Einkaufsliste
    elif action == "getshoppinglist":
        response = shopping_list.get_shopping_list(user, db)
    elif action == "addeditshoppinglistitem":
        response = shopping_list.add_edit_shopping_list_item(user, request['id'],
                                                             request['amount'], request['name'], db)
    elif action == "deleteshoppinglistitem":
        response = shopping_list.delete_shopping_list_item(user, request['id'], db)

    # Sprachassistent
    elif action == "voicecommand":
        response = VoiceAssistant().do_voice_command(user, request['text'], user_last_location, None, db, language)
        # response = do_voice_command(user, request['text'], user_last_location, None, db, language)
    elif action == "getvoicereplaceitems":
        response = get_voice_replace_items(user, db)
    elif action == "addeditvoicereplaceitem":
        response = add_edit_voice_replace_item(user, request['replacewith'], request['itemstoreplace'], db)
    elif action == "deletevoicereplaceitem":
        response = delete_voice_replace_item(user, request['replacewith'], db)

    # Smart Speakers
    elif action == "getsmartspeakers":
        response = get_smart_speakers(user, db)

    # Heizpläne
    elif action == "addeditheatingschemeitem":
        response = add_edit_heating_scheme_item(user, request['id'], request['time'], request['value'],
                                                request['active'], request['days'], request['data'], db)
    elif action == "deleteheatingschemeitem":
        response = delete_heating_scheme_item(user, request['id'], db)
    elif action == "getheatingschemeitems":
        response = get_heating_scheme_items(user, request['day'], request['rooms'], db)
    elif action == "setheatingschemeitemactive":
        response = set_heating_scheme_item_active(user, request['id'], request['active'], db)
    elif action == "getheatingschemeitemdata":
        response = get_heating_scheme_item_data(user, request['id'], db)
    elif action == "setheatingschemeactive":
        response = set_heating_scheme_active(user, request['active'], db)
    elif action == "isheatingschemeactive":
        response = is_heating_scheme_active(user, db)

    # API-Keys
    elif action == "getapikeydata":
        response = get_all_api_key_data(user, db)
    elif action == "setapikey":
        response = set_api_key(user, request['service'], request['apikey'], db)

    # Media Center
    elif action == "getmediacenters":
        response = media_center.get_media_centers(user, db)
    elif action == "mediacentercontrol":
        response = media_center.media_remote_action(request['type'], request['id'], request['remoteaction'], db)
    elif action == "mediacentersendtext":
        response = media_center.media_center_send_text(request['type'], request['id'], request['text'], db)
    elif action == "mediacentermusic":
        response = media_center.get_media_center_music(request['type'], request['id'],
                                                       request['limit'], request['offset'], db)
    elif action == "mediacenterartists":
        response = media_center.get_media_center_artists(request['type'], request['id'],
                                                         request['limit'], request['offset'], db)
    elif action == "mediacenteralbums":
        response = media_center.get_media_center_albums(request['type'], request['id'],
                                                        request['limit'], request['offset'], db)
    elif action == "mediacentermusicgenres":
        response = media_center.get_media_center_music_genres(request['type'], request['id'],
                                                              request['limit'], request['offset'], db)
    elif action == "mediacentertvshows":
        response = media_center.get_media_center_shows(request['type'], request['id'],
                                                       request['limit'], request['offset'], db)
    elif action == "mediacentertvshowseasons":
        response = media_center.get_media_center_show_seasons(request['type'], request['id'], request['limit'],
                                                              request['offset'], request['showid'], db)
    elif action == "mediacentertvshowepisodes":
        response = media_center.get_media_center_show_episodes(request['type'], request['id'], request['limit'],
                                                               request['offset'], request['showid'],
                                                               request['seasonid'], db)
    elif action == "mediacentermovies":
        response = media_center.get_media_center_movies(request['type'], request['id'],
                                                        request['limit'], request['offset'], db)
    elif action == "mediacentermoviegenres":
        response = media_center.get_media_center_movie_genres(request['type'], request['id'],
                                                              request['limit'], request['offset'], db)
    elif action == "mediacenterplaying":
        response = media_center.get_media_menter_playing(request['type'], request['id'], db)

    # Firebase-Token aktualisieren
    elif action == "updatefcmtoken":
        response = set_user_fcm_token(user, request['token'], db)

    # Heizung
    elif action == "heatingcontrol":
        response = heating_control(user, request['type'], request['id'], request['value'], db)
    elif action == "controlroomheating":
        response = control_room_heating(user, request['location'], request['value'], db)
    elif action == "getthermostats":
        response = get_thermostats(user, request['room'], request['type'], request['id'], db)

    # Räume
    elif action == "addeditroom":
        response = add_edit_room(user, request['roomname'], request['location'], request['icon'], db)
    elif action == "moveitemsanddeleteoldroom":
        response = move_items_and_delete_old_room(user, request['oldroom'], request['newroom'], db)
    elif action == "deleteroomwithitems":
        response = delete_room_with_items(user, request['location'], db)

    # Geräte-Manager
    elif action == "addeditdevice":
        response = add_edit_device(user, request['id'], request['type'], request['data'], db)
    elif action == "deletedevice":
        response = delete_device(user, request['type'], request['id'], db)
    elif action == "getdevicedata":
        response = get_device_data(user, request['type'], request['id'], db)

    # AR-Control
    elif action == "arcontrol":
        response = ar_control.ar_control(user, request['imagedata'], db)
    elif action == "getarcontrolclasses":
        response = ar_control.get_ar_control_classes(db)
    elif action == "savearcontrolclass":
        response = ar_control.save_ar_control_class(user, request['id'], request['data'], request['classname'], db)
    elif action == "getarcontrolclassimages":
        response = ar_control.get_ar_control_class_images(request['class'], request['show'], request['offset'], db)
    elif action == "startarcontroltraining":
        response = ar_control.start_ar_training(user, db)
    elif action == "changearcontrolimageclass":
        response = ar_control.change_ar_image_class(request['ids'], request['newclass'], db)
    elif action == "getarcontrolimages":
        response = ar_control.get_ar_control_image(request['id'], db)
    elif action == "getarimageclassifierperformancesettings":
        response = ar_control.get_performance_settings(user, db)
    elif action == "setarimageclassifierperformancesettings":
        response = ar_control.set_performance_settings(user, request['data'], db)
    elif action == "uploadarcontrolimages":
        response = ar_control.upload_images(user, request['data'], request['class'], db)
    elif action == "deletearcontrolimages":
        response = ar_control.delete_ar_control_images(request['ids'], db)

    # Gesichtserkennung
    elif action == "getpeopleclasses":
        response = people_classifier.get_people_classes(db)
    elif action == "savepeopleclass":
        response = people_classifier.save_people_class(user, request['id'], request['data'], request['classname'], db)
    elif action == "startpeopletraining":
        response = people_classifier.start_people_training(user, db)
    elif action == "getpeopleimageclassifierperformancesettings":
        response = people_classifier.get_performance_settings(user, db)
    elif action == "setpeopleimageclassifierperformancesettings":
        response = people_classifier.set_performance_settings(user, request['data'], db)
    elif action == "changepeopleclassifierimageclass":
        response = people_classifier.change_people_image_class(request['ids'], request['newclass'], db)
    elif action == "getpeopleclassifierimage":
        response = people_classifier.get_people_image(request['id'], db)
    elif action == "getpeopleclassifierimages":
        response = people_classifier.get_people_images(request['class'], request['show'], request['offset'], db)
    elif action == "uploadpeopleimages":
        response = people_classifier.upload_images(user, request['data'], request['class'], db)
    elif action == "classifyperson":
        response = people_classifier.classify_person(request['imagedata'], db)
    elif action == "deletepeopleimages":
        response = people_classifier.delete_people_images(request['ids'], db)

    # Automation
    elif action == "getautomationrules":
        response = get_automations(user, request['location'], db)
    elif action == "addeditautomationrule":
        response = add_edit_automation_rule(user, request['location'], request['id'], request['name'],
                                            request['triggerdata'], request['conditiondata'], request['actiondata'],
                                            request['isactive'], db)
    elif action == "deleteautomationrule":
        response = delete_automation_rule(user, request['id'], db)
    # Eigene Sprachbefehle
    elif action == "getvoicecommands":
        response = get_voice_commands(db)
    elif action == "addeditvoicecommand":
        response = add_edit_voice_command(user, request['id'], request['name'], request['command_data'],
                                          request['response_data'], request['action_data'], db)
    elif action == "deletevoicecommand":
        response = delete_voice_command(user, request['id'], db)
    # Szenen
    elif action == "getallscenes":
        response = get_all_scenes(user, db)
    elif action == "getscenes":
        response = get_scenes(user, request['location'], db)
    elif action == "addeditscene":
        response = add_edit_scene(user, request['id'], request['name'], request['location'], request['action_data'], db)
    elif action == "deletescene":
        response = delete_scene(user, request['id'], db)
    elif action == "runscene":
        response = run_scene(user, request['id'], db)
    # Haushaltskasse
    elif action == "gethomebudgetdata":
        response = get_home_budget_data(user, request['startdate'], db)
    elif action == "gethomebudgetdatadayitems":
        response = get_home_budget_data_day_items(user, request['date'], db)
    elif action == "gethomebudgetdatagraph":
        response = get_home_budget_data_graph(user, request['startdate'], request['enddate'], db)
    elif action == "addedithomebudgetdata":
        response = add_edit_home_budget_data(user, request['id'], request['date'], request['info'], request['amount'],
                                             db)
    elif action == "deletehomebudgetdata":
        response = delete_home_budget_data(user, request['id'], db)
    # GPS data
    elif action == "updategps":
        response = gps_data.update_gps(user, request['time'], request['lat'], request['lng'], db)
    elif action == "getgpslocations":
        response = gps_data.get_gps_locations(user, db)
    # Places
    elif action == "addeditplace":
        response = places.add_edit_place(user, request['id'], request['name'], request['address'], request['latitude'],
                                         request['longitude'], db)
    elif action == "getmyplaces":
        response = places.get_my_places(user, db)
    elif action == "deleteplace":
        response = places.delete_place(user, request['id'], db)
    # Ernährungsmanager
    elif action == "submitfood":
        response = nutrition_data.submit_food(user, request['name'], request['calories'], request['portionsize'],
                                              request['portionunit'], request['protein'], request['fat'],
                                              request['saturated'], request['unsaturated'], request['carbs'],
                                              request['sugar'], request['ean'], db)
    elif action == "getusernutritionoverview":
        response = nutrition_data.get_user_nutrition_overview(user, db)
    elif action == "getuserdaynutritionitems":
        response = nutrition_data.get_user_day_nutrition_items(user, request['date'], db)
    elif action == "addedituserdaynutritionitem":
        response = nutrition_data.add_edit_user_day_nutrition_item(user, request['id'], request['date'],
                                                                   request['daytime'], request['name'],
                                                                   request['eatenportionsize'], request['portionsize'],
                                                                   request['portionunit'],
                                                                   request['calories'], request['fat'],
                                                                   request['saturated'], request['unsaturated'],
                                                                   request['carbs'], request['sugar'],
                                                                   request['protein'], db)
    elif action == "loadnutritionmanagersettings":
        response = nutrition_data.load_user_settings(user, db)
    elif action == "deletefooditem":
        response = nutrition_data.delete_food_item(user, request['id'], db)
    elif action == "savenutritionmanagersettings":
        response = nutrition_data.save_user_settings(user, request['height'], request['weight'], request['birthdate'],
                                                     request['mode'], request['activity'], db)
    elif action == "movenutritionitem":
        response = nutrition_data.move_item(user, request['id'], request['date'], request['daytime'],
                                            request['deleteold'], db)
    # Updates
    elif action == "checkforupdates":
        response = Updater.check_for_updates()
    elif action == "updatesystem":
        response = Updater.do_homevee_update(user, db)
    else:
        return {'result': 'nosuchaction'}

    # Ausgeben
    return json.dumps(response)

    '''
    //Kalender
        case "getcalendaritems":
            $result = getCalendarItems($_POST['cal_user'], $db);
            break;
        case "getcalendaritemsforday":
            $result = getCalendarItemsForDay($_POST['USERNAME'], $_POST['date'], $db);
            break;
        case "getcalendardataformonth":
            $result = getCalendarDataForMonth($_POST['USERNAME'], $_POST['month'], $db);
            break;
        case "getcalendariteminfo":
            $result = getCalendarItemInfo($_POST['id'], $db);
            break;
        case "addeditcalendaritem":
            $result = addEditCalendarItem($_POST['id'], $_POST['name'],
            $_POST['start'], $_POST['end'], $_POST['note'], $_POST['repeat'], $_POST['participants'], $db);
            break;
        case "deletecalendaritem":
            $result = deleteCalendarItem($_POST['id'], $db);
            break;
    '''

    '''
        case "runscene":
            $result = runScene($_POST['room'], $_POST['name'], $db);
            break;
        case "createscene":
            $result = createScene($_POST['devices'], $_POST['rooms'], $_POST['types'], $_POST['values'],
				$_POST['conditions'], $_POST['room'], $_POST['name'], $db);
            break;
        case "getscenes":
            $result = getScenes($_POST['room'], $db);
            break;

		//Noch nicht im Tutorial
		case "shouldshowmorninginfo":
			//$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
			break;
		case "getmorninginfo":
			//$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
			break;
		case "getmorninginfosettings":
			//$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
			break;
		case "setmorninginfosettings":
			//$result = iotDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
			break;
		case "rfidvalidation":
			$result = rfidValidation($_POST['uid'], $db);
			break;
		case "rfidtrigger":
			$result = rfidTrigger($_POST['uid'], $_POST['triggerid'], $db);
			break;
		case "heatingcontrol":
			$result = heatingControl($_POST['room'], $_POST['type'], $_POST['id'], $_POST['value'], $db);
			break;
		case "getheatingdata":
			$result = getHeatingData($_POST['room'], $_POST['type'], $_POST['id'], $_POST['all_data'], $db);
			break;
		case "controldiydevice":
			$result = diyDeviceControl($_POST['device'], $_POST['room'], $_POST['diyaction'], $_POST['value'], $db);
			break;
		case "getdiyinfo":
			$result = diyDeviceControl($_POST['device'], $_POST['room'], 'getinfo', "", $db);
			break;
		case "getreeddata":
			$result = getReedData($_POST['room'], $_POST['type'], $_POST['id'], $db);
			break;
		case "addedittargetid":
			$result = addEditTargetID($_POST['user'], $_POST['deviceid'], $_POST['targetid'], $db);
			break;
        case "getpowermeterdata":
            $result = getSensorData($_POST['room'], $_POST['type'], $_POST['id'], $_POST['hide_einheit'], $db);
            break;
        case "getdimmer":
            $result = getDimmer($_POST['room'], $_POST['type'], $_POST['id'], $db);
            break;
        case "setdimmer":
            $result = setDimmer($_POST['room'], $_POST['type'], $_POST['id'], $_POST['value'], $db);
            break;

		//Stromverbrauch
		case "getenergydata":
			$result = getEnergyData($_POST['room'], $_POST['devicetype'], $_POST['deviceid'], $_POST['von'], $_POST['bis'], $db);
			break;
		case "getdeviceenergydata":
			$result = getDeviceEnergyData($_POST['type'], $_POST['device'], $_POST['von'], $_POST['bis'], $db);
			break;
		case "setpowercost":
			$result = setPowerCost($_POST['cost'], $db);
			break;
		case "getpowercost":
			$result = getPowerCost($db);
			break;

		//Überwachungskamera
		case "getsurveillancefootage":
			$result = getSurveillanceFootage($_POST['id'], $_POST['filterdata'], $_POST['offset'], $_POST['limit'], $db);
			break;
		case "getsurveillancethumbnail":
			$result = getSurveillanceThumbnail($_POST['id'], $db);
			break;
		case "getsurveillancefootagevideo":
			$result = getSurveillanceFootageVideo($_POST['id'], $db);
			break;
		case "getipcameradata":
			$result = getIpCameraData($_POST['room'], $_POST['id'], $db);
			break;

		//Türspion
		case "addpeepholeimages":
			$result = addPeepholeImages($_POST['imagedata'], $db);
			break;
		case "getpeepholeclassimages":
			$result = getPeepholeClassImages($_POST['class'], $_POST['show'], $_POST['offset'], $db);
			break;
		case "changepeepholeimageclass":
			$result = changePeepholeClass($_POST['ids'], $_POST['newclass'], $db);
			break;
		case "getpeepholeimage":
			$result = getPeepholeImage($_POST['id'], $db);
			break;
		case "addeditpeepholeclass":
			$result = addEditPeepHoleClass($_POST['id'], $_POST['name'], $db);
			break;
		case "deletepeepholeclass":
			$result = deletePeepHoleClass($_POST['id'], $db);
			break;

		//Bild-Klassifizierer Einstellungen
		case "getimageclassifierperformancesettings":
			$result = getImageClassifierPerformanceSettings($db);
			break;
		case "setimageclassifierperformancesettings":
			$result = setImageClassifierPerformanceSettings($_POST['data'], $db);
			break;

		//Manager
		case "getdevicedata":
			$result = getDeviceData($_POST['type'], $_POST['id'], $db);
			break;
		case "deletedevice":
			$result = deleteDevice($_POST['type'], $_POST['id'], $db);
			break;
		case "addeditdevice":
			$result = addEditDevice($_POST['id'], $_POST['type'], $_POST['data'], $db);
			break;

		//Kalender
		case "getcalendaritems":
			$result = getCalendarItems($_POST['cal_user'], $db);
			break;
		case "getcalendaritemsforday":
			$result = getCalendarItemsForDay($_POST['USERNAME'], $_POST['date'], $db);
			break;
		case "getcalendardataformonth":
			$result = getCalendarDataForMonth($_POST['USERNAME'], $_POST['month'], $db);
			break;
		case "getcalendariteminfo":
			$result = getCalendarItemInfo($_POST['id'], $db);
			break;
		case "addeditcalendaritem":
			$result = addEditCalendarItem($_POST['id'], $_POST['name'],
			$_POST['start'], $_POST['end'], $_POST['note'], $_POST['repeat'], $_POST['participants'], $db);
			break;
		case "deletecalendaritem":
			$result = deleteCalendarItem($_POST['id'], $db);
			break;

		//MediaCenter
		case "getmediacenters":
			$result = getMediaCenters($db);
			break;
		case "mediacentercontrol":
			$result = remoteAction($_POST['type'], $_POST['id'], $_POST['remoteaction'], $db);
			break;
		case "mediacentersendtext":
			$result = mediaCenterSendText($_POST['type'], $_POST['id'], $_POST['text'], $db);
			break;
		case "mediacentermusic":
			$result = getMediaCenterMusic($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
			break;
		case "mediacenterartists":
			$result = getMediaCenterArtists($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
			break;
		case "mediacenteralbums":
			$result = getMediaCenterAlbums($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
			break;
		case "mediacentermusicgenres":
			$result = getMediaCenterMusicGenres($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
			break;
		case "mediacentertvshows":
			$result = getMediaCenterShows($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
			break;
		case "mediacentertvshowseasons":
			$result = getMediaCenterShowSeasons($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $_POST['showid'], $db);
			break;
		case "mediacentertvshowepisodes":
			$result = getMediaCenterShowEpisodes($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $_POST['showid'], $_POST['seasonid'], $db);
			break;
		case "mediacentermovies":
			$result = getMediaCenterMovies($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
			break;
		case "mediacentermoviegenres":
			$result = getMediaCenterMovieGenres($_POST['type'], $_POST['id'], $_POST['limit'], $_POST['offset'], $db);
			break;
		case "mediacenterplaying":
			$result = getMediaCenterPlaying($_POST['type'], $_POST['id'], $db);
			break;

		//Update
		case "checkforupdates":
			$result = checkForUpdates($db);
			break;
		case "updatesystem":
			$result = updateSystem($db);
			break;
        '''


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.DeviceAPI.zwave.get_devices import get_device_value
from homevee.Helper.helper_functions import has_permission
from homevee.items.Device.Thermostat.MaxThermostat import *
from homevee.items.Device.Thermostat.RademacherThermostat import *
from homevee.items.Device.Thermostat.ZWaveThermostat import *
from homevee.items.Status import *
from homevee.utils.device_types import *


def heating_control(user, type, id, value, db, check_user=True):
    if type == MAX_CUBE:
        thermostat = MaxThermostat.load_from_db(id, db)
    elif type == ZWAVE_THERMOSTAT:
        thermostat = ZWaveThermostat.load_from_db(id, db)
    elif type == RADEMACHER_THERMOSTAT:
        thermostat = RademacherThermostat.load_from_db(id, db)
    else:
        return Status(type=STATUS_NO_SUCH_TYPE).get_dict()

    if check_user and not user.has_permission(thermostat.location):
        return Status(type=STATUS_NO_PERMISSION).get_dict()

    if(thermostat.set_temp(value)):
        print("result == True")
        return Status(type=STATUS_OK).get_dict()
    else:
        print("result == False")
        return Status(type=STATUS_ERROR).get_dict()

    '''
    try:
        with db:
            cur = db.cursor()

            if type == ZWAVE_THERMOSTAT:
                cur.execute("SELECT RAUM FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", {'id': id})
                data = cur.fetchone()
                if check_user and not has_permission(username, data['RAUM'], db):
                    return {'result': 'nopermission'}

                #update value in db as well
                set_device_value(type, id, value, db)

                result = do_zwave_request("/ZAutomation/api/v1/devices/"+str(id)+"/command/exact?level="+str(value), db)

                if result is not None and result['code'] == 200:
                    return Status(type=STATUS_OK).get_dict()
                else:
                    return Status(type=STATUS_ERROR).get_dict()
            elif type == MAX_THERMOSTAT:
                thermostat = MaxThermostat.load_from_db(id, db)
                if(thermostat.set_temp(value)):
                    return Status(type=STATUS_OK).get_dict()
                else:
                    return Status(type=STATUS_ERROR).get_dict()

                #cur.execute("SELECT RAUM FROM MAX_THERMOSTATS WHERE ID = :id", {'id': id})
                #data = cur.fetchone()
                #if check_user and not has_permission(username, data['RAUM'], db):
                #    return {'result': 'nopermission'}
                #return set_temp(id, value, db)
            elif type == RADEMACHER_THERMOSTAT:
                cur.execute("SELECT ROOM FROM HOMEPILOT_THERMOSTATS WHERE ID = :id", {'id': id})
                data = cur.fetchone()
                if check_user and not has_permission(username, data['ROOM'], db):
                    return Status(type=STATUS_NO_PERMISSION).get_dict()

                return rademacher_homepilot.heating_control.heating_control(id, value, db)
            else:
                return Status(type=STATUS_NO_SUCH_TYPE).get_dict()
    except:
        if(Logger.IS_DEBUG):
                traceback.print_exc()
        return Status(type=STATUS_ERROR).get_dict()
    '''

def control_room_heating(username, room, value, db):
    with db:
        devices = get_thermostats(username, room, "", "", db)['thermostats'][0]['thermostat_array']
        for device in devices:
            id = device['id']
            type = device['type']
            result = heating_control(username, type, id, value, db)
            if 'result' not in result or result['result'] != "ok":
                return Status(type=STATUS_ERROR).get_dict()
        return Status(type=STATUS_OK).get_dict()

def get_thermostats(username, room, type, id, db):
    with db:
        cur = db.cursor()

        if type != "" and id != "":
            #if not has_permission(username, room, db):
            #   return {'result': 'nopermission'}

            if type == ZWAVE_THERMOSTAT:
                '''
                cur.execute("SELECT RAUM FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID = :id", {'id': id})
                data = cur.fetchone()
                if not has_permission(username, data['RAUM'], db):
                    return {'result': 'nopermission'}

                result = do_zwave_request("/ZAutomation/api/v1/devices/"+id, db)

                if result['code'] != 200:
                    return "N/A"

                return {'value': result['data']['metrics']['level'],
                        'min': result['data']['metrics']['min'],
                        'max': result['data']['metrics']['max']}
                '''

                value = get_device_value(type, id, db)
                return {'value': value, 'min': 4, 'max': 40}

            elif type == MAX_THERMOSTAT:
                cur.execute("SELECT * FROM MAX_THERMOSTATS WHERE ID == :id",
                            {'id': id})

                result = cur.fetchone()

                return {'value': result['LAST_TEMP'], 'min': 0, 'max': 30}

            elif type == RADEMACHER_THERMOSTAT:
                cur.execute("SELECT * FROM HOMEPILOT_THERMOSTATS WHERE ID == :id",
                            {'id': id})

                result = cur.fetchone()

                return {'value': result['LAST_TEMP'], 'min': 4, 'max': 28}

            else:
                return {'result': 'nosuchtype'}
        else:
            if room == "all":
                cur.execute("SELECT * FROM ROOMS")
            else:
                cur.execute("SELECT * FROM ROOMS WHERE LOCATION == :room",
                            {'room': room})

            rooms = cur.fetchall()

            thermostats = []

            for room in rooms:
                if not has_permission(username, room['LOCATION'], db):
                    continue

                thermostat_array = []

                # Alle Z-Wave Thermostate des Raumes laden
                type = ZWAVE_THERMOSTAT
                cur.execute("SELECT * FROM ZWAVE_THERMOSTATS WHERE RAUM == :location",
                            {'location': room['LOCATION']})
                results = cur.fetchall()
                for result in results:
                    thermostat = {'name': result['NAME'], 'id': result['THERMOSTAT_ID'],
                                  'type': type, 'icon': result['ICON'],
                                  'data': get_thermostats(username, room['LOCATION'], type, result['THERMOSTAT_ID'], db)}
                    thermostat_array.append(thermostat)

                # Alle MAX! Thermostate des Raumes laden
                type = MAX_THERMOSTAT
                cur.execute("SELECT * FROM MAX_THERMOSTATS WHERE RAUM == :location",
                            {'location': room['LOCATION']})
                results = cur.fetchall()
                for result in results:
                    thermostat = {'name': result['NAME'], 'id': result['ID'],
                                  'type': type, 'icon': result['ICON'],
                                  'data': get_thermostats(username, room['LOCATION'], type, result['ID'], db)}
                    thermostat_array.append(thermostat)

                # Alle Rademacher Thermostate des Raumes laden
                type = RADEMACHER_THERMOSTAT
                cur.execute("SELECT * FROM HOMEPILOT_THERMOSTATS WHERE ROOM == :location",
                            {'location': room['LOCATION']})
                results = cur.fetchall()
                for result in results:
                    thermostat = {'name': result['NAME'], 'id': result['ID'],
                                  'type': type, 'icon': result['ICON'],
                                  'data': get_thermostats(username, room['LOCATION'], type, result['ID'], db)}
                    thermostat_array.append(thermostat)

                room_thermostats = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'],
                                    'thermostat_array': thermostat_array}
                thermostats.append(room_thermostats)

            return {'thermostats': thermostats}



from homevee.VoiceAssistant import VoiceAssistant

def user_is_at_home(username, db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM USERDATA WHERE USERNAME = :username", {'username': username})

        user = cur.fetchone()

        return user['AT_HOME']

def do_voice_command(username, text, user_last_location, location, db, language):
    return VoiceAssistant().do_voice_command(username, text, user_last_location, location, db, language)


from homevee.items.Device import Device


class RGBLight(Device):
    def __init__(self, name, icon, location, id=None, mode=False):
        super(RGBLight, self).__init__(name, icon, location, id=id)


#!/usr/bin/python
# -*- coding: utf-8 -*-

import datetime
import re

from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.helper import generate_string
from homevee.VoiceAssistant.voice_patterns import PATTERN_DATE_WEEKDAY


class VoiceGetWeekdayModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_DATE_WEEKDAY

    def get_label(self):
        return "getweekday"

    def run_command(self, username, text, context, db):
        return self.get_weekday(username, text, context, db)

    def get_weekday(self, username, text, context, db):
        wochentage = ['Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag', 'Sonntag']

        month_replacements = [
            ['. januar ', '.01.'],
            ['. februar ', '.02.'],
            ['. märz ', '.03.'],
            ['. april ', '.04.'],
            ['. mai ', '.05.'],
            ['. juni ', '.06.'],
            ['. juli ', '.07.'],
            ['. august ', '.08.'],
            ['. september ', '.09.'],
            ['. oktober ', '.10.'],
            ['. november ', '.11.'],
            ['. dezember ', '.12.'],
        ]

        for replacement in month_replacements:
            text = text.replace(replacement[0], replacement[1])

        date = re.search('\d{2}\.\d{2}\.\d{4}', text)
        if(date is None):
            answer_data = [
                ['Du hast kein Datum ', ['gesagt', 'genannt'],'.'],
                ['Du musst', [' mir ', ' '], ['zuerst', 'erst', 'erst einmal'], ' ein Datum ', ['nennen', 'sagen'], '.']
            ]

            output = generate_string(answer_data)
        else:
            date_formatted = str(datetime.datetime.strptime(date.group(), '%d.%m.%Y').date())

            date_data = date_formatted.split('-')
            time = datetime.datetime(int(date_data[0]), int(date_data[1]), int(date_data[2]))
            wochentag = wochentage[time.weekday()]

            answer_data = [
                [[['Der ', date_formatted], 'Das'], ' ist ein ', wochentag, [', '+username, ''], '.']
            ]

            output = generate_string(answer_data)

        return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import random
import urllib.error
import urllib.error
import urllib.parse
import urllib.parse
import urllib.request
import urllib.request

from homevee.Helper import Logger
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.Modules import VoiceModule
from homevee.VoiceAssistant.voice_patterns import PATTERN_ACTIVITY


class VoiceGetActivitiesModule(VoiceModule):
    def get_pattern(self, db):
        return PATTERN_ACTIVITY

    def get_label(self):
        return "activity"

    def run_command(self, username, text, context, db):
        return self.get_activities(username, text, context, db)

    def get_activities(self, username, text, context, db):
        try:
            url = helper.SMART_API_PATH + "/?action=activity&text=" + urllib.parse.quote(text.encode('utf8'))
            Logger.log(url)

            response = urllib.request.urlopen(url)
            data = response.read()

            data = data.decode('utf-8')

            if data is not None:
                return {'msg_speech': data, 'msg_text': data}
            else:
                output = self.get_error()
        except urllib.error.HTTPError as e:
            output = self.get_error()

        return {'msg_speech': output, 'msg_text': output}

    def get_error(self):
        return random.choice([
            'Mir fällt gerade nichts ein, was du tun könntest.'
        ])


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.cronjobs import IntervalCronjob
from homevee.utils.database import get_database_con

class CalendarReminderCronjob(IntervalCronjob):
    def __init__(self):
        super(CalendarReminderCronjob, self).__init__(task_name="CalendarReminder", interval_seconds=60)

    def task_to_do(self, *args):
        self.remind_users()

    def remind_users(self):
        db = get_database_con()

        with db:
            #query events and remind users of upcoming calendar entries

            return


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import socket
import ssl
import time
import traceback
from _thread import start_new_thread
from urllib.parse import urlencode
from urllib.request import Request, urlopen

from homevee.Helper import Logger, translations
from homevee.utils.constants import END_OF_MESSAGE
from homevee.utils.database import get_database_con, set_server_data
from .API import process_data
from .Helper.helper_functions import remote_control_enabled

CLOUD_URL = "premium.cloud.homevee.de"
CLOUD_PORT = 7778

#Verbindung zu Cloud herstellen

#Auf Befehl warten, ausführen und Ergebnis zurück an Server senden

class CloudConnection():
    def __init__(self):
        self.cloud_url = CLOUD_URL
        self.cloud_port = CLOUD_PORT

    def start_connection_loop(self):
        start_new_thread(self.connection_loop, ())

    def connection_loop(self):
        db = get_database_con()
        REMOTE_ID, ACCESS_TOKEN = self.get_remote_data(db)
        if REMOTE_ID is None or ACCESS_TOKEN is None:
            REMOTE_ID, ACCESS_TOKEN = self.register_to_cloud(db)
            # Logger.log("NO CLOUD CONNECTION CREDENTIALS")

        print(translations.translate('your_remote_id_is') + REMOTE_ID)

        while (True):
            self.do_connection()

    def do_connection(self):
        db = get_database_con()
        conn = None

        if (not remote_control_enabled(db)):
            # Logger.log("REMOTE CONTROL NOT ENABLED")
            # time.sleep(5 * 60)  # 5 Minuten warten
            time.sleep(10)
            return

        cloud_to_use = self.get_cloud_to_use(db)

        if cloud_to_use is not None:
            #Logger.log("using cloud: " + cloud_to_use)
            pass
        else:
            cloud_to_use = 'free.cloud.homevee.de'
            #Logger.log("using default cloud: " + cloud_to_use)
            pass

        try:
            conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # conn.settimeout(5*60)
            # conn.connect((CLOUD_URL, CLOUD_PORT))
            # Logger.log("Connected")

            '''
            conn = ssl.wrap_socket(conn,
                                         ca_certs=SSL_FULLCHAIN,
                                         cert_reqs=ssl.CERT_REQUIRED,
                                         ssl_version=ssl.PROTOCOL_SSLv23)
            '''

            # SSL aktivieren
            conn = ssl.wrap_socket(conn)

            # Logger.log("Wrapping SSL")
            # Logger.log(("Connecting to cloud ("+CLOUD_URL+")..."))
            conn.connect((cloud_to_use, CLOUD_PORT))
            start_new_thread(self.keep_connection_alive, (conn, 60))
            # print ("SSL connected")
            # Logger.log("Connection successful!")

            REMOTE_ID, ACCESS_TOKEN = self.get_remote_data(db)
            data_dict = {'access_token': ACCESS_TOKEN, 'remote_id': REMOTE_ID}
            data = json.dumps(data_dict) + END_OF_MESSAGE

            len_send = conn.send(bytes(data, 'utf-8'))

            while 1:
                try:
                    data = conn.recv(8192).decode('utf-8')
                except:
                    data = None

                if data is None or data == "":
                    db.close()
                    conn.close()
                    return

                Logger.log("")
                Logger.log(("Received from Cloud: " + data))

                data_parts = data.split(END_OF_MESSAGE)

                for data_part in data_parts:
                    if data_part is None or data_part == "":
                        continue

                    Logger.log("")
                    Logger.log(data_part)

                    data_dict = json.loads(data_part)

                    if 'result' in data_dict:
                        Logger.log(data_dict)

                        if (data_dict['result'] == "error"):
                            Logger.log("couldn't connect to cloud")
                            time.sleep(60)  # 1 Minute warten
                    else:
                        #Logger.log("Processing data:")
                        #Logger.log(data_dict)
                        try:
                            client_id = data_dict['client_id']

                            data_dict = json.loads(data_dict['msg'])

                            msg = process_data(data_dict, db)

                            # start_time = time.time()
                            # compressed_message = compression.compress_string(msg)
                            # end_time = time.time()

                            # print "uncompressed: " + str(len(msg))
                            # print "compressed: " + str(len(compressed_message)) + ", time: " + str(end_time - start_time)
                            # compressed_message = compressed_message.decode('iso-8859-1').encode('utf8')

                            data = {}
                            data['msg'] = msg
                            data['client_id'] = client_id
                            data = json.dumps(data)

                            if data is None:
                                data = {'status': 'error'}
                                data['client_id'] = client_id
                                data = json.dumps(data)
                            # else:
                            #    data['client_id'] = client_id
                            #    data['msg'] = json.dumps(msg)
                            #    data = json.dumps(data)

                            data_to_send = data + END_OF_MESSAGE

                            len_send = conn.send(data_to_send.encode('utf-8'))

                            Logger.log(("Sent to remote Server (" + data_dict['action'] + ")(client: " + str(
                                client_id) + "): " + data))
                        except Exception as e:
                            #if(Logger.IS_DEBUG):
                            #    traceback.print_exc()
                            if 'status' in data_dict:
                                if data_dict['status'] == 'connectionok':
                                    #Logger.log('connectionok')
                                    pass
                                if data_dict['status'] == 'nocredentials':
                                    # Credentials wrong
                                    # deleting them from db
                                    with db:
                                        # Logger.log("wrongcredentials")
                                        time.sleep(60)  # 1 Minute warten
                                        # cur = db.cursor()
                                        # cur.execute("DELETE FROM SERVER_DATA WHERE KEY IN ('REMOTE_ID', 'REMOTE_ACCESS_TOKEN')")

        except:
            # do not print errors
            if(Logger.IS_DEBUG):
                traceback.print_exc()
            time.sleep(5)

            if conn is not None:
                conn.close()

        db.close()

    def keep_connection_alive(self, conn, time):
        while True:
            try:
                # Send some data every minute
                time.sleep(time)
                Logger.log("sending ping")
                conn.send(1)
            except:
                break

    # Returns a tuple: (REMOTE_ID, PRIVATE_KEY)
    def get_remote_data(self, db):
        with db:
            cur = db.cursor()

            # cur.execute("DELETE FROM SERVER_DATA WHERE KEY IN ('REMOTE_ACCESS_TOKEN', 'REMOTE_ID')")

            cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = 'REMOTE_ID'")

            result = cur.fetchone()

            if (result is None):
                return None, None
            else:
                remote_id = result['VALUE']
                # Logger.log(("Die Remote-ID lautet: "+remote_id))

            cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = 'REMOTE_ACCESS_TOKEN'")

            result = cur.fetchone()

            if (result is None):
                return None, None
            else:
                remote_access_token = result['VALUE']
                # Logger.log(("Access Token: "+remote_access_token))

        return (remote_id, remote_access_token)

    def get_cloud_to_use(self, db):
        try:
            REMOTE_ID, ACCESS_TOKEN = self.get_remote_data(db)

            url = 'https://cloud.homevee.de/server-api.php'  # Set destination URL here
            post_fields = {'action': 'getcloudtouse',
                           'remoteid': REMOTE_ID,
                           'accesstoken': ACCESS_TOKEN}  # Set POST fields here

            # Logger.log(url+"?action=getcloudtouse&remoteid="+REMOTE_ID+"&accesstoken="+ACCESS_TOKEN)

            request = Request(url, urlencode(post_fields).encode())
            response = urlopen(request).read().decode()

            data = json.loads(response)
            return data['cloud']
        except:
            return None

    def register_to_cloud(self, db):
        url = 'https://cloud.homevee.de/server-api.php'  # Set destination URL here
        post_fields = {'action': 'generateremoteid'}  # Set POST fields here

        request = Request(url, urlencode(post_fields).encode())
        response = urlopen(request).read().decode()

        data = json.loads(response)

        remote_id = data['remote_id']
        access_token = data['access_token']

        # Logger.log("Deine Remote-ID lautet: "+remote_id)

        # save remote id
        set_server_data("REMOTE_ID", remote_id, db)
        set_server_data("REMOTE_ACCESS_TOKEN", access_token, db)

        return remote_id, access_token


#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import os

from homevee.utils.constants import DATA_DIR


def get_image_directory_path(directory_name):
	path = os.path.join(DATA_DIR, "images", directory_name)

	return path

def create_image(filename, directory_name, img_data, optimize=False):
	filename = filename + ".jpeg"

	rel_path = os.path.join("images", directory_name)

	abs_path = os.path.join(DATA_DIR, rel_path)

	if not os.path.exists(abs_path):
		os.makedirs(abs_path)

	file_path = os.path.join(abs_path, filename)

	#Logger.log(img_data)

	img_data = base64.b64decode(img_data)

	fh = open(file_path, "wb")
	fh.write(img_data)
	fh.close()

	if(optimize):
		optimize_image(file_path)

	return os.path.join(rel_path, filename)

def optimize_image(file_path):
	return


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from homevee.Helper.translations import german
from homevee.utils import constants

LANGUAGE = "en"
#LANGUAGE = "de"

def translate(key, language=None):
    translations = {
        'en': get_translations(),
        'de': german.get_translations()
    }

    if language is None or language not in translations:
        language = LANGUAGE

    return translations[language][key]

def get_translations():
    translations = {
        'no_users_create_admin': "You have not created any users yet.\nYou can create an administrator-account now.",
        'enter_password': 'Please enter a password: ',
        'password_dont_match': 'The given passwords dont match',
        'enter_username': 'Please enter a username: ',
        'your_remote_id_is': 'Your remote-id is: ',
        'homevee_server_started': "Homevee-Server (Version: " + constants.HOMEVEE_VERSION_NUMBER + ") has been started...",
        'update_available': "There's an update for Homevee available ({})",
        'no_cloud_connection': "Could not connect to Homevee-cloud..."
    }
    return translations


#!/usr/bin/python
# -*- coding: utf-8 -*-

CONTEXT_SET_MODES = "CONTEXT_SET_MODES"
CONTEXT_GET_SENSOR_DATA = "CONTEXT_GET_SENSOR_DATA"
CONTEXT_SET_HEATING = "CONTEXT_SET_HEATING"
CONTEXT_SET_THERMOSTAT = "CONTEXT_SET_THERMOSTAT"
CONTEXT_CONTROL_BLINDS = "CONTEXT_CONTROL_BLINDS"
CONTEXT_GET_WEATHER = "CONTEXT_GET_WEATHER"
CONTEXT_ADD_NUTRITION = "CONTEXT_ADD_NUTRITION"
CONTEXT_TV_PROGRAMME = "CONTEXT_TV_PROGRAMME"
CONTEXT_WOL = "CONTEXT_WOL"
CONTEXT_CALCULATOR = "CONTEXT_CALCULATOR"


from homevee.exceptions import ItemNotFoundException, InvalidParametersException, DatabaseSaveFailedException
from homevee.items import Item
from homevee.utils import database


class Room(Item):
    def __init__(self, name, icon, id=None):
        super(Room, self).__init__()
        self.id = id
        self.name = name
        self.icon = icon

    def get_room_data(self, db):
        if(self.room_data is None):
            room_data = None #TODO load room data

            self.room_data = room_data

        return self.room_data

    def delete(self, db=None):
        #TODO implement room deletion
        pass

    def save_to_db(self, db=None):
        try:
            if(db is None):
                db = database.get_database_con()

            with db:
                cur = db.cursor()
                #insert
                if(self.id is None or self.id == ""):
                    cur.execute("INSERT INTO ROOMS (NAME, ICON) VALUES (:name, :icon)",
                                {'name': self.name, 'icon': self.icon})
                #update
                else:
                    cur.execute("UPDATE ROOMS SET NAME = :name, ICON = :icon WHERE LOCATION = :id",
                                {'name': self.name, 'icon': self.icon, 'id': self.id})

                cur.close()

                    # TODO add generated id to object
        except:
            raise DatabaseSaveFailedException("Could not save room to database")

    def get_dict(self, fields=None):
        dict = {
            'id': self.id,
            'name': self.name,
            'icon': self.icon
        }

        if(fields is None):
            return dict
        else:
            try:
                output_dict = {}

                for field in fields:
                    output_dict[field] = dict[field]

                return output_dict
            except:
                raise InvalidParametersException("InvalidParams given for Room.get_dict()")

    @staticmethod
    def load_all_ids_from_db(ids, db=None):
        return Room.load_all_from_db('SELECT * FROM ROOMS WHERE LOCATION IN (%s)' % ','.join('?'*len(ids)),
                                     ids, db)

    @staticmethod
    def load_all_from_db(query, params, db=None):
        if (db is None):
            db = database.get_database_con()

        items = []

        with db:
            cur = db.cursor()

            cur.execute(query, params)

            for result in cur.fetchall():
                item = Room(result['NAME'], result['ICON'], result['LOCATION'])
                items.append(item)

            cur.close()

        return items

    @staticmethod
    def get_name_by_id(id, db=None):
        room = Room.load_from_db(id, db)
        return room.name

    @staticmethod
    #ID is username
    def load_from_db(id, db=None):
        items = Room.load_all_ids_from_db([id], db)

        if((len(items) == 0) or (str(items[0].id) != str(id))):
            raise ItemNotFoundException("Could not find room with id: " + str(id))
        else:
            return items[0]

    @staticmethod
    def load_all(db=None):
        return Room.load_all_from_db('SELECT * FROM ROOMS', {}, db)

    @staticmethod
    def create_from_dict(dict):
        try:
            id = dict['id']
            name = dict['name']
            icon = dict['icon']

            room = Room(name, icon, id)

            return room
        except:
            raise InvalidParametersException("Room.create_from_dict(): invalid dict")


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.DeviceAPI import philips_hue
from homevee.cronjobs import IntervalCronjob
from homevee.utils.database import get_database_con

class PhilipsHueValueLoaderCronjob(IntervalCronjob):
    def __init__(self):
        super(PhilipsHueValueLoaderCronjob, self).__init__(task_name="PhilipsHueValueLoaderCronjob", interval_seconds=5*60)

    def task_to_do(self, *args):
        self.load_hue_values()

    def load_hue_values(self):
        db = get_database_con()

        with db:
            cur = db.cursor()
            cur.execute("SELECT * FROM PHILIPS_HUE_LIGHTS")
            for item in cur.fetchall():
                counter = 3
                for i in range(0, counter):
                    if(philips_hue.get_light_info(item['ID'], db)):
                        break

        db.close()


#!/usr/bin/env python
# Copyright 2015 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#		 http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
r"""Simple transfer learning with Inception v3 or Mobilenet models.
With support for TensorBoard.
This example shows how to take a Inception v3 or Mobilenet model trained on
ImageNet images, and train a new top layer that can recognize other classes of
images.
The top layer receives as input a 2048-dimensional vector (1001-dimensional for
Mobilenet) for each image. We train a softmax layer on top of this
representation. Assuming the softmax layer contains N labels, this corresponds
to learning N + 2048*N (or 1001*N)	model parameters corresponding to the
learned biases and weights.
Here's an example, which assumes you have a folder containing class-named
subfolders, each full of images for each label. The example folder flower_photos
should have a structure like this:
~/flower_photos/daisy/photo1.jpg
~/flower_photos/daisy/photo2.jpg
...
~/flower_photos/rose/anotherphoto77.jpg
...
~/flower_photos/sunflower/somepicture.jpg
The subfolder names are important, since they define what label is applied to
each image, but the filenames themselves don't matter. Once your images are
prepared, you can run the training with a command like this:
```bash
bazel build tensorflow_data/examples/image_retraining:retrain && \
bazel-bin/tensorflow_data/examples/image_retraining/retrain \
		--image_dir ~/flower_photos
```
Or, if you have a pip installation of tensorflow_data, `retrain.py` can be run
without bazel:
```bash
python tensorflow_data/examples/image_retraining/retrain.py \
		--image_dir ~/flower_photos
```
You can replace the image_dir argument with any folder containing subfolders of
images. The label for each image is taken from the name of the subfolder it's
in.
This produces a new model file that can be loaded and run by any TensorFlow
program, for example the label_image sample code.
By default this script will use the high accuracy, but comparatively large and
slow Inception v3 model architecture. It's recommended that you start with this
to validate that you have gathered good training data, but if you want to deploy
on resource-limited platforms, you can try the `--architecture` flag with a
Mobilenet model. For example:
```bash
python tensorflow_data/examples/image_retraining/retrain.py \
		--image_dir ~/flower_photos --architecture mobilenet_1.0_224
```
There are 32 different Mobilenet models to choose from, with a variety of file
size and latency options. The first number can be '1.0', '0.75', '0.50', or
'0.25' to control the size, and the second controls the input image size, either
'224', '192', '160', or '128', with smaller sizes running faster. See
https://research.googleblog.com/2017/06/mobilenets-open-source-models-for.html
for more information on Mobilenet.
To use with TensorBoard:
By default, this script will log summaries to /tmp/retrain_logs directory
Visualize the summaries with this command:
tensorboard --logdir /tmp/retrain_logs
"""




import hashlib
import os
import os.path
import random
import re
import sys
import tarfile
import traceback
from datetime import datetime

import numpy as np

from homevee.Helper import Logger
from homevee.utils.file_utils import get_image_directory_path

try:
    import tensorflow as tf
    from tensorflow.python.framework import graph_util
    from tensorflow.python.framework import tensor_shape
    from tensorflow.python.platform import gfile
    from tensorflow.python.util import compat
    TENSORFLOW_INSTALLED = True
    #Logger.log("Tensorflow imported")
except:
    TENSORFLOW_INSTALLED = False
    Logger.log("Error importing Tensorflow")

from homevee.utils import constants
from homevee.utils.database import get_database_con, get_server_data
from homevee.utils.firebase_utils import send_notification_to_users

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
image_dir = get_image_directory_path("ar_control")
output_graph = os.path.join(BASE_DIR, "tensorflow_data", "people_temp", "output_graph.pb")
intermediate_output_graphs_dir = os.path.join(BASE_DIR, "people_temp", "ar_control_temp", "intermediate_graph")
intermediate_store_frequency = 0
output_labels = os.path.join(BASE_DIR, "tensorflow_data", "people_temp", "output_labels.txt")
summaries_dir = os.path.join(BASE_DIR, "tensorflow_data", "people_temp", "retrain_logs")
how_many_training_steps = 500
learning_rate = 0.01
testing_percentage = 10
validation_percentage = 10
eval_step_interval = 10
train_batch_size = 100
test_batch_size = -1
validation_batch_size = 100
print_misclassified_test_images = False
model_dir = os.path.join(BASE_DIR, "tensorflow_data", "models")
bottleneck_dir = os.path.join(BASE_DIR, "tensorflow_data", "people_temp", "bottleneck")
final_tensor_name = "final_result"
flip_left_right = False
random_crop = 0
random_scale = 0
random_brightness = 0
architecture = "mobilenet_1.0_224"

# These are all parameters that are tied to the particular model architecture
# we're using for Inception v3. These include things like tensor names and their
# sizes. If you want to adapt this script to work with another model, you will
# need to update these to reflect the values in the network you're using.
MAX_NUM_IMAGES_PER_CLASS = 2 ** 27 - 1  # ~134M


def create_image_lists(image_dir, testing_percentage, validation_percentage):
    result = {}

    db = get_database_con()

    labels = []
    with db:
        cur = db.cursor()
        cur.execute("SELECT DISTINCT(PERSON_ID) FROM (SELECT * FROM PEOPLE_LEARNING_DATA WHERE PERSON_ID IS NOT NULL)")

        for data in cur.fetchall():
            Logger.log(data)
            labels.append(data['PERSON_ID'])

        Logger.log("Labels: " + str(labels))

        for label in labels:
            cur = db.cursor()
            cur.execute("SELECT * FROM PEOPLE_LEARNING_DATA WHERE PERSON_ID = :key", {"key": label})

            file_list = []

            for data in cur.fetchall():
                file_list.extend(gfile.Glob(os.path.join(constants.DATA_DIR, data['PATH'])))
                Logger.log("Pfad: " + str(os.path.join(constants.DATA_DIR, data['PATH'])))

            if len(file_list) < 3:
                continue

            training_images = []
            testing_images = []
            validation_images = []
            for file_name in file_list:
                base_name = os.path.basename(file_name)
                # We want to ignore anything after '_nohash_' in the file name when
                # deciding which set to put an image in, the data set creator has a way of
                # grouping photos that are close variations of each other. For example
                # this is used in the plant disease data set to group multiple pictures of
                # the same leaf.
                hash_name = re.sub(r'_nohash_.*$', '', file_name)
                # This looks a bit magical, but we need to decide whether this file should
                # go into the training, testing, or validation sets, and we want to keep
                # existing files in the same set even if more files are subsequently
                # added.
                # To do that, we need a stable way of deciding based on just the file name
                # itself, so we do a hash of that and then use that to generate a
                # probability value that we use to assign it.
                hash_name_hashed = hashlib.sha1(compat.as_bytes(hash_name)).hexdigest()
                percentage_hash = ((int(hash_name_hashed, 16) %
                                    (MAX_NUM_IMAGES_PER_CLASS + 1)) *
                                   (100.0 / MAX_NUM_IMAGES_PER_CLASS))
                if percentage_hash < validation_percentage:
                    validation_images.append(base_name)
                elif percentage_hash < (testing_percentage + validation_percentage):
                    testing_images.append(base_name)
                else:
                    training_images.append(base_name)

            if len(validation_images) == 0:
                if len(testing_images) > len(training_images) and len(training_images) >= 1:
                    validation_images.append(testing_images[0])
                    del testing_images[0]
                else:
                    validation_images.append(training_images[0])
                    del training_images[0]

            if len(training_images) == 0:
                if len(testing_images) > len(validation_images) and len(validation_images) >= 1:
                    training_images.append(testing_images[0])
                    del testing_images[0]
                else:
                    training_images.append(validation_images[0])
                    del validation_images[0]

            if len(testing_images) == 0:
                if len(validation_images) > len(training_images) and len(training_images) >= 1:
                    testing_images.append(validation_images[0])
                    del validation_images[0]
                else:
                    testing_images.append(training_images[0])
                    del training_images[0]

            label = str(label)

            Logger.log(label)
            Logger.log("Validation: " + str(len(validation_images)))
            Logger.log("Testing: " + str(len(testing_images)))
            Logger.log("Training: " + str(len(training_images)))

            result[label] = {
                'dir': os.path.join(constants.DATA_DIR, "images", "people"),
                'training': training_images,
                'testing': testing_images,
                'validation': validation_images,
            }

        Logger.log(result)
    return result


def get_image_path(image_lists, label_name, index, image_dir, category):
    """"Returns a path to an image for a label at the given index.
    Args:
        image_lists: Dictionary of training images for each label.
        label_name: Label string we want to get an image for.
        index: Int offset of the image we want. This will be moduloed by the
        available number of images for the label, so it can be arbitrarily large.
        image_dir: Root folder string of the subfolders containing the training
        images.
        category: Name string of set to pull images from - training, testing, or
        validation.
    Returns:
        File system path string to an image that meets the requested parameters.
    """
    if label_name not in image_lists:
        tf.logging.fatal('Label does not exist %s.', label_name)
    label_lists = image_lists[label_name]
    if category not in label_lists:
        tf.logging.fatal('Category does not exist %s.', category)
    category_list = label_lists[category]
    if not category_list:
        tf.logging.fatal('Label %s has no images in the category %s.',
                         label_name, category)

    mod_index = index % len(category_list)

    base_name = category_list[mod_index]
    sub_dir = label_lists['dir']
    full_path = os.path.join(image_dir, sub_dir, base_name)
    return full_path


def get_bottleneck_path(image_lists, label_name, index, bottleneck_dir,
                        category, architecture):
    """"Returns a path to a bottleneck file for a label at the given index.
    Args:
        image_lists: Dictionary of training images for each label.
        label_name: Label string we want to get an image for.
        index: Integer offset of the image we want. This will be moduloed by the
        available number of images for the label, so it can be arbitrarily large.
        bottleneck_dir: Folder string holding cached files of bottleneck values.
        category: Name string of set to pull images from - training, testing, or
        validation.
        architecture: The name of the model architecture.
    Returns:
        File system path string to an image that meets the requested parameters.
    """
    return get_image_path(image_lists, label_name, index, bottleneck_dir, category) + '_' + architecture + '.txt'


def create_model_graph(model_info):
    """"Creates a graph from saved GraphDef file and returns a Graph object.
    Args:
        model_info: Dictionary containing information about the model architecture.
    Returns:
        Graph holding the trained Inception network, and various tensors we'll be
        manipulating.
    """
    with tf.Graph().as_default() as graph:
        model_path = os.path.join(model_dir, model_info['model_file_name'])
        with gfile.FastGFile(model_path, 'rb') as f:
            graph_def = tf.GraphDef()
            graph_def.ParseFromString(f.read())
            bottleneck_tensor, resized_input_tensor = (tf.import_graph_def(
                graph_def,
                name='',
                return_elements=[
                    model_info['bottleneck_tensor_name'],
                    model_info['resized_input_tensor_name'],
                ]))
    return graph, bottleneck_tensor, resized_input_tensor


def run_bottleneck_on_image(sess, image_data, image_data_tensor, decoded_image_tensor, resized_input_tensor,
                            bottleneck_tensor):
    """Runs inference on an image to extract the 'bottleneck' summary layer.
    Args:
        sess: Current active TensorFlow Session.
        image_data: String of raw JPEG data.
        image_data_tensor: Input data layer in the graph.
        decoded_image_tensor: Output of initial image resizing and	preprocessing.
        resized_input_tensor: The input node of the recognition graph.
        bottleneck_tensor: Layer before the final softmax.
    Returns:
        Numpy array of bottleneck values.
    """
    # First decode the JPEG image, resize it, and rescale the pixel values.
    resized_input_values = sess.run(decoded_image_tensor, {image_data_tensor: image_data})
    # Then run it through the recognition network.
    bottleneck_values = sess.run(bottleneck_tensor, {resized_input_tensor: resized_input_values})
    bottleneck_values = np.squeeze(bottleneck_values)
    return bottleneck_values


def maybe_download_and_extract(data_url):
    """Download and extract model tar file.
    If the pretrained model we're using doesn't already exist, this function
    downloads it from the TensorFlow.org website and unpacks it into a directory.
    Args:
        data_url: Web location of the tar file containing the pretrained model.
    """
    dest_directory = model_dir
    if not os.path.exists(dest_directory):
        os.makedirs(dest_directory)
    filename = data_url.split('/')[-1]
    filepath = os.path.join(dest_directory, filename)
    if not os.path.exists(filepath):
        def _progress(count, block_size, total_size):
            sys.stdout.write(
                '\r>> Downloading %s %.1f%%' % (filename, float(count * block_size) / float(total_size) * 100.0))
            sys.stdout.flush()

        statinfo = os.stat(filepath)
        tf.logging.info('Successfully downloaded', filename, statinfo.st_size,
                        'bytes.')
    tarfile.open(filepath, 'r:gz').extractall(dest_directory)


def ensure_dir_exists(dir_name):
    """Makes sure the folder exists on disk.
    Args:
        dir_name: Path string to the folder we want to create.
    """
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)


bottleneck_path_2_bottleneck_values = {}


def create_bottleneck_file(bottleneck_path, image_lists, label_name, index, image_dir, category, sess, jpeg_data_tensor,
                           decoded_image_tensor, resized_input_tensor,
                           bottleneck_tensor):
    """Create a single bottleneck file."""
    tf.logging.info('Creating bottleneck at ' + bottleneck_path)
    image_path = get_image_path(image_lists, label_name, index,
                                image_dir, category)
    if not gfile.Exists(image_path):
        tf.logging.fatal('File does not exist %s', image_path)
    image_data = gfile.FastGFile(image_path, 'rb').read()
    try:
        bottleneck_values = run_bottleneck_on_image(
            sess, image_data, jpeg_data_tensor, decoded_image_tensor,
            resized_input_tensor, bottleneck_tensor)
    except Exception as e:
        raise RuntimeError('Error during processing file %s (%s)' % (image_path, str(e)))
    bottleneck_string = ','.join(str(x) for x in bottleneck_values)
    with open(bottleneck_path, 'w') as bottleneck_file:
        bottleneck_file.write(bottleneck_string)


def get_or_create_bottleneck(sess, image_lists, label_name, index, image_dir,
                             category, bottleneck_dir, jpeg_data_tensor,
                             decoded_image_tensor, resized_input_tensor,
                             bottleneck_tensor, architecture):
    """Retrieves or calculates bottleneck values for an image.
    If a cached version of the bottleneck data exists on-disk, return that,
    otherwise calculate the data and save it to disk for future use.
    Args:
        sess: The current active TensorFlow Session.
        image_lists: Dictionary of training images for each label.
        label_name: Label string we want to get an image for.
        index: Integer offset of the image we want. This will be modulo-ed by the
        available number of images for the label, so it can be arbitrarily large.
        image_dir: Root folder string	of the subfolders containing the training
        images.
        category: Name string of which	set to pull images from - training, testing,
        or validation.
        bottleneck_dir: Folder string holding cached files of bottleneck values.
        jpeg_data_tensor: The tensor to feed loaded jpeg data into.
        decoded_image_tensor: The output of decoding and resizing the image.
        resized_input_tensor: The input node of the recognition graph.
        bottleneck_tensor: The output tensor for the bottleneck values.
        architecture: The name of the model architecture.
    Returns:
        Numpy array of values produced by the bottleneck layer for the image.
    """
    label_lists = image_lists[label_name]
    sub_dir = label_lists['dir']
    sub_dir_path = os.path.join(bottleneck_dir, sub_dir)
    ensure_dir_exists(sub_dir_path)
    bottleneck_path = get_bottleneck_path(image_lists, label_name, index,
                                          bottleneck_dir, category, architecture)
    if not os.path.exists(bottleneck_path):
        create_bottleneck_file(bottleneck_path, image_lists, label_name, index,
                               image_dir, category, sess, jpeg_data_tensor,
                               decoded_image_tensor, resized_input_tensor,
                               bottleneck_tensor)
    with open(bottleneck_path, 'r') as bottleneck_file:
        bottleneck_string = bottleneck_file.read()
    did_hit_error = False
    try:
        bottleneck_values = [float(x) for x in bottleneck_string.split(',')]
    except ValueError:
        tf.logging.warning('Invalid float found, recreating bottleneck')
        did_hit_error = True
    if did_hit_error:
        create_bottleneck_file(bottleneck_path, image_lists, label_name, index,
                               image_dir, category, sess, jpeg_data_tensor,
                               decoded_image_tensor, resized_input_tensor,
                               bottleneck_tensor)
        with open(bottleneck_path, 'r') as bottleneck_file:
            bottleneck_string = bottleneck_file.read()
        # Allow exceptions to propagate here, since they shouldn't happen after a
        # fresh creation
        bottleneck_values = [float(x) for x in bottleneck_string.split(',')]
    return bottleneck_values


def cache_bottlenecks(sess, image_lists, image_dir, bottleneck_dir,
                      jpeg_data_tensor, decoded_image_tensor,
                      resized_input_tensor, bottleneck_tensor, architecture):
    """Ensures all the training, testing, and validation bottlenecks are cached.
    Because we're likely to read the same image multiple times (if there are no
    distortions applied during training) it can speed things up a lot if we
    calculate the bottleneck layer values once for each image during
    preprocessing, and then just read those cached values repeatedly during
    training. Here we go through all the images we've found, calculate those
    values, and save them off.
    Args:
        sess: The current active TensorFlow Session.
        image_lists: Dictionary of training images for each label.
        image_dir: Root folder string of the subfolders containing the training
        images.
        bottleneck_dir: Folder string holding cached files of bottleneck values.
        jpeg_data_tensor: Input tensor for jpeg data from file.
        decoded_image_tensor: The output of decoding and resizing the image.
        resized_input_tensor: The input node of the recognition graph.
        bottleneck_tensor: The penultimate output layer of the graph.
        architecture: The name of the model architecture.
    Returns:
        Nothing.
    """
    how_many_bottlenecks = 0
    ensure_dir_exists(bottleneck_dir)
    for label_name, label_lists in list(image_lists.items()):
        for category in ['training', 'testing', 'validation']:
            category_list = label_lists[category]
            for index, unused_base_name in enumerate(category_list):
                get_or_create_bottleneck(
                    sess, image_lists, label_name, index, image_dir, category,
                    bottleneck_dir, jpeg_data_tensor, decoded_image_tensor,
                    resized_input_tensor, bottleneck_tensor, architecture)

                how_many_bottlenecks += 1
                if how_many_bottlenecks % 100 == 0:
                    tf.logging.info(
                        str(how_many_bottlenecks) + ' bottleneck files created.')


def get_random_cached_bottlenecks(sess, image_lists, how_many, category, bottleneck_dir,
                                  image_dir, jpeg_data_tensor, decoded_image_tensor, resized_input_tensor,
                                  bottleneck_tensor, architecture):
    """Retrieves bottleneck values for cached images.
    If no distortions are being applied, this function can retrieve the cached
    bottleneck values directly from disk for images. It picks a random set of
    images from the specified category.
    Args:
        sess: Current TensorFlow Session.
        image_lists: Dictionary of training images for each label.
        how_many: If positive, a random sample of this size will be chosen.
        If negative, all bottlenecks will be retrieved.
        category: Name string of which set to pull from - training, testing, or
        validation.
        bottleneck_dir: Folder string holding cached files of bottleneck values.
        image_dir: Root folder string of the subfolders containing the training
        images.
        jpeg_data_tensor: The layer to feed jpeg image data into.
        decoded_image_tensor: The output of decoding and resizing the image.
        resized_input_tensor: The input node of the recognition graph.
        bottleneck_tensor: The bottleneck output layer of the CNN graph.
        architecture: The name of the model architecture.
    Returns:
        List of bottleneck arrays, their corresponding ground truths, and the
        relevant filenames.
    """
    class_count = len(list(image_lists.keys()))
    bottlenecks = []
    ground_truths = []
    filenames = []
    if how_many >= 0:
        # Retrieve a random sample of bottlenecks.
        for unused_i in range(how_many):
            label_index = random.randrange(class_count)
            label_name = list(image_lists.keys())[label_index]
            image_index = random.randrange(MAX_NUM_IMAGES_PER_CLASS + 1)
            image_name = get_image_path(image_lists, label_name, image_index, image_dir, category)
            bottleneck = get_or_create_bottleneck(
                sess, image_lists, label_name, image_index, image_dir, category,
                bottleneck_dir, jpeg_data_tensor, decoded_image_tensor,
                resized_input_tensor, bottleneck_tensor, architecture)
            ground_truth = np.zeros(class_count, dtype=np.float32)
            ground_truth[label_index] = 1.0
            bottlenecks.append(bottleneck)
            ground_truths.append(ground_truth)
            filenames.append(image_name)
    else:
        # Retrieve all bottlenecks.
        for label_index, label_name in enumerate(image_lists.keys()):
            for image_index, image_name in enumerate(
                    image_lists[label_name][category]):
                image_name = get_image_path(image_lists, label_name, image_index, image_dir, category)
                bottleneck = get_or_create_bottleneck(
                    sess, image_lists, label_name, image_index, image_dir, category,
                    bottleneck_dir, jpeg_data_tensor, decoded_image_tensor,
                    resized_input_tensor, bottleneck_tensor, architecture)
                ground_truth = np.zeros(class_count, dtype=np.float32)
                ground_truth[label_index] = 1.0
                bottlenecks.append(bottleneck)
                ground_truths.append(ground_truth)
                filenames.append(image_name)
    return bottlenecks, ground_truths, filenames


def get_random_distorted_bottlenecks(
        sess, image_lists, how_many, category, image_dir, input_jpeg_tensor,
        distorted_image, resized_input_tensor, bottleneck_tensor):
    """Retrieves bottleneck values for training images, after distortions.
    If we're training with distortions like crops, scales, or flips, we have to
    recalculate the full model for every image, and so we can't use cached
    bottleneck values. Instead we find random images for the requested category,
    run them through the distortion graph, and then the full graph to get the
    bottleneck results for each.
    Args:
        sess: Current TensorFlow Session.
        image_lists: Dictionary of training images for each label.
        how_many: The integer number of bottleneck values to return.
        category: Name string of which set of images to fetch - training, testing,
        or validation.
        image_dir: Root folder string of the subfolders containing the training
        images.
        input_jpeg_tensor: The input layer we feed the image data to.
        distorted_image: The output node of the distortion graph.
        resized_input_tensor: The input node of the recognition graph.
        bottleneck_tensor: The bottleneck output layer of the CNN graph.
    Returns:
        List of bottleneck arrays and their corresponding ground truths.
    """
    class_count = len(list(image_lists.keys()))
    bottlenecks = []
    ground_truths = []
    for unused_i in range(how_many):
        label_index = random.randrange(class_count)
        label_name = list(image_lists.keys())[label_index]
        image_index = random.randrange(MAX_NUM_IMAGES_PER_CLASS + 1)
        image_path = get_image_path(image_lists, label_name, image_index, image_dir, category)
        if not gfile.Exists(image_path):
            tf.logging.fatal('File does not exist %s', image_path)
        jpeg_data = gfile.FastGFile(image_path, 'rb').read()
        # Note that we materialize the distorted_image_data as a numpy array before
        # sending running inference on the image. This involves 2 memory copies and
        # might be optimized in other implementations.
        distorted_image_data = sess.run(distorted_image, {input_jpeg_tensor: jpeg_data})
        bottleneck_values = sess.run(bottleneck_tensor, {resized_input_tensor: distorted_image_data})
        bottleneck_values = np.squeeze(bottleneck_values)
        ground_truth = np.zeros(class_count, dtype=np.float32)
        ground_truth[label_index] = 1.0
        bottlenecks.append(bottleneck_values)
        ground_truths.append(ground_truth)
    return bottlenecks, ground_truths


def should_distort_images(flip_left_right, random_crop, random_scale,
                          random_brightness):
    """Whether any distortions are enabled, from the input
    Args:
        flip_left_right: Boolean whether to randomly mirror images horizontally.
        random_crop: Integer percentage setting the total margin used around the
        crop box.
        random_scale: Integer percentage of how much to vary the scale by.
        random_brightness: Integer range to randomly multiply the pixel values by.
    Returns:
        Boolean value indicating whether any distortions should be applied.
    """
    return (flip_left_right or (random_crop != 0) or (random_scale != 0) or
            (random_brightness != 0))


def add_input_distortions(flip_left_right, random_crop, random_scale,
                          random_brightness, input_width, input_height,
                          input_depth, input_mean, input_std):
    """Creates the operations to apply the specified distortions.
    During training it can help to improve the results if we run the images
    through simple distortions like crops, scales, and flips. These reflect the
    kind of variations we expect in the real world, and so can help train the
    model to cope with natural data more effectively. Here we take the supplied
    parameters and construct a network of operations to apply them to an image.
    Cropping
    ~~~~~~~~
    Cropping is done by placing a bounding box at a random position in the full
    image. The cropping parameter controls the size of that box relative to the
    input image. If it's zero, then the box is the same size as the input and no
    cropping is performed. If the value is 50%, then the crop box will be half the
    width and height of the input. In a diagram it looks like this:
    <			 width				 >
    +---------------------+
    |										 |
    |	 width - crop%		 |
    |		<			>				 |
    |		+------+				 |
    |		|			|				 |
    |		|			|				 |
    |		|			|				 |
    |		+------+				 |
    |										 |
    |										 |
    +---------------------+
    Scaling
    ~~~~~~~
    Scaling is a lot like cropping, except that the bounding box is always
    centered and its size varies randomly within the given range. For example if
    the scale percentage is zero, then the bounding box is the same size as the
    input and no scaling is applied. If it's 50%, then the bounding box will be in
    a random range between half the width and height and full size.
    Args:
        flip_left_right: Boolean whether to randomly mirror images horizontally.
        random_crop: Integer percentage setting the total margin used around the
        crop box.
        random_scale: Integer percentage of how much to vary the scale by.
        random_brightness: Integer range to randomly multiply the pixel values by.
        graph.
        input_width: Horizontal size of expected input image to model.
        input_height: Vertical size of expected input image to model.
        input_depth: How many channels the expected input image should have.
        input_mean: Pixel value that should be zero in the image for the graph.
        input_std: How much to divide the pixel values by before recognition.
    Returns:
        The jpeg input layer and the distorted result tensor.
    """

    jpeg_data = tf.placeholder(tf.string, name='DistortJPGInput')
    decoded_image = tf.image.decode_jpeg(jpeg_data, channels=input_depth)
    decoded_image_as_float = tf.cast(decoded_image, dtype=tf.float32)
    decoded_image_4d = tf.expand_dims(decoded_image_as_float, 0)
    margin_scale = 1.0 + (random_crop / 100.0)
    resize_scale = 1.0 + (random_scale / 100.0)
    margin_scale_value = tf.constant(margin_scale)
    resize_scale_value = tf.random_uniform(tensor_shape.scalar(), minval=1.0, maxval=resize_scale)
    scale_value = tf.multiply(margin_scale_value, resize_scale_value)
    precrop_width = tf.multiply(scale_value, input_width)
    precrop_height = tf.multiply(scale_value, input_height)
    precrop_shape = tf.stack([precrop_height, precrop_width])
    precrop_shape_as_int = tf.cast(precrop_shape, dtype=tf.int32)
    precropped_image = tf.image.resize_bilinear(decoded_image_4d, precrop_shape_as_int)
    precropped_image_3d = tf.squeeze(precropped_image, squeeze_dims=[0])
    cropped_image = tf.random_crop(precropped_image_3d,
                                   [input_height, input_width, input_depth])
    if flip_left_right:
        flipped_image = tf.image.random_flip_left_right(cropped_image)
    else:
        flipped_image = cropped_image
    brightness_min = 1.0 - (random_brightness / 100.0)
    brightness_max = 1.0 + (random_brightness / 100.0)
    brightness_value = tf.random_uniform(tensor_shape.scalar(), minval=brightness_min, maxval=brightness_max)
    brightened_image = tf.multiply(flipped_image, brightness_value)
    offset_image = tf.subtract(brightened_image, input_mean)
    mul_image = tf.multiply(offset_image, 1.0 / input_std)
    distort_result = tf.expand_dims(mul_image, 0, name='DistortResult')
    return jpeg_data, distort_result


def variable_summaries(var):
    """Attach a lot of summaries to a Tensor (for TensorBoard visualization)."""
    with tf.name_scope('summaries'):
        mean = tf.reduce_mean(var)
        tf.summary.scalar('mean', mean)
        with tf.name_scope('stddev'):
            stddev = tf.sqrt(tf.reduce_mean(tf.square(var - mean)))
        tf.summary.scalar('stddev', stddev)
        tf.summary.scalar('max', tf.reduce_max(var))
        tf.summary.scalar('min', tf.reduce_min(var))
        tf.summary.histogram('histogram', var)


def add_final_training_ops(class_count, final_tensor_name, bottleneck_tensor, bottleneck_tensor_size):
    """Adds a new softmax and fully-connected layer for training.
    We need to retrain the top layer to identify our new classes, so this function
    adds the right operations to the graph, along with some variables to hold the
    weights, and then sets up all the gradients for the backward pass.
    The set up for the softmax and fully-connected layers is based on:
    https://www.tensorflow_data.org/versions/master/tutorials/mnist/beginners/index.html
    Args:
        class_count: Integer of how many categories of things we're trying to
        recognize.
        final_tensor_name: Name string for the new final node that produces results.
        bottleneck_tensor: The output of the main CNN graph.
        bottleneck_tensor_size: How many entries in the bottleneck vector.
    Returns:
        The tensors for the training and cross entropy results, and tensors for the
        bottleneck input and ground truth input.
    """
    with tf.name_scope('input'):
        bottleneck_input = tf.placeholder_with_default(
            bottleneck_tensor,
            shape=[None, bottleneck_tensor_size],
            name='BottleneckInputPlaceholder')

        ground_truth_input = tf.placeholder(tf.float32, [None, class_count], name='GroundTruthInput')

    # Organizing the following ops as `final_training_ops` so they're easier
    # to see in TensorBoard
    layer_name = 'final_training_ops'
    with tf.name_scope(layer_name):
        with tf.name_scope('weights'):
            initial_value = tf.truncated_normal(
                [bottleneck_tensor_size, class_count], stddev=0.001)

            layer_weights = tf.Variable(initial_value, name='final_weights')

            variable_summaries(layer_weights)
        with tf.name_scope('biases'):
            layer_biases = tf.Variable(tf.zeros([class_count]), name='final_biases')
            variable_summaries(layer_biases)
        with tf.name_scope('Wx_plus_b'):
            logits = tf.matmul(bottleneck_input, layer_weights) + layer_biases
            tf.summary.histogram('pre_activations', logits)

    final_tensor = tf.nn.softmax(logits, name=final_tensor_name)
    tf.summary.histogram('activations', final_tensor)

    with tf.name_scope('cross_entropy'):
        cross_entropy = tf.nn.softmax_cross_entropy_with_logits(
            labels=ground_truth_input, logits=logits)
        with tf.name_scope('total'):
            cross_entropy_mean = tf.reduce_mean(cross_entropy)
    tf.summary.scalar('cross_entropy', cross_entropy_mean)

    with tf.name_scope('train'):
        optimizer = tf.train.GradientDescentOptimizer(learning_rate)
        train_step = optimizer.minimize(cross_entropy_mean)

    return (train_step, cross_entropy_mean, bottleneck_input, ground_truth_input,
            final_tensor)


def add_evaluation_step(result_tensor, ground_truth_tensor):
    """Inserts the operations we need to evaluate the accuracy of our results.
    Args:
        result_tensor: The new final node that produces results.
        ground_truth_tensor: The node we feed ground truth data
        into.
    Returns:
        Tuple of (evaluation step, prediction).
    """
    with tf.name_scope('accuracy'):
        with tf.name_scope('correct_prediction'):
            prediction = tf.argmax(result_tensor, 1)
            correct_prediction = tf.equal(
                prediction, tf.argmax(ground_truth_tensor, 1))
        with tf.name_scope('accuracy'):
            evaluation_step = tf.reduce_mean(tf.cast(correct_prediction, tf.float32))
    tf.summary.scalar('accuracy', evaluation_step)
    return evaluation_step, prediction


def save_graph_to_file(sess, graph, graph_file_name):
    output_graph_def = graph_util.convert_variables_to_constants(
        sess, graph.as_graph_def(), [final_tensor_name])
    with gfile.FastGFile(graph_file_name, 'wb') as f:
        f.write(output_graph_def.SerializeToString())
    return


def prepare_file_system():
    # Setup the directory we'll write summaries to for TensorBoard
    if tf.gfile.Exists(summaries_dir):
        tf.gfile.DeleteRecursively(summaries_dir)
    tf.gfile.MakeDirs(summaries_dir)
    if intermediate_store_frequency > 0:
        ensure_dir_exists(intermediate_output_graphs_dir)
    return


def create_model_info(architecture):
    """Given the name of a model architecture, returns information about it.
    There are different base image recognition pretrained models that can be
    retrained using transfer learning, and this function translates from the name
    of a model to the attributes that are needed to download and train with it.
    Args:
        architecture: Name of a model architecture.
    Returns:
        Dictionary of information about the model, or None if the name isn't
        recognized
    Raises:
        ValueError: If architecture name is unknown.
    """
    architecture = architecture.lower()
    if architecture == 'inception_v3':
        # pylint: disable=line-too-long
        data_url = 'http://download.tensorflow_data.org/models/image/imagenet/inception-2015-12-05.tgz'
        # pylint: enable=line-too-long
        bottleneck_tensor_name = 'pool_3/_reshape:0'
        bottleneck_tensor_size = 2048
        input_width = 299
        input_height = 299
        input_depth = 3
        resized_input_tensor_name = 'Mul:0'
        model_file_name = 'classify_image_graph_def.pb'
        input_mean = 128
        input_std = 128
    elif architecture.startswith('mobilenet_'):
        parts = architecture.split('_')
        if len(parts) != 3 and len(parts) != 4:
            tf.logging.error("Couldn't understand architecture name '%s'",
                             architecture)
            return None
        version_string = parts[1]
        if (version_string != '1.0' and version_string != '0.75' and
                version_string != '0.50' and version_string != '0.25'):
            tf.logging.error(
                """"The Mobilenet version should be '1.0', '0.75', '0.50', or '0.25',
but found '%s' for architecture '%s'""",
                version_string, architecture)
            return None
        size_string = parts[2]
        if (size_string != '224' and size_string != '192' and
                size_string != '160' and size_string != '128'):
            tf.logging.error(
                """The Mobilenet input size should be '224', '192', '160', or '128',
but found '%s' for architecture '%s'""",
                size_string, architecture)
            return None
        if len(parts) == 3:
            is_quantized = False
        else:
            if parts[3] != 'quantized':
                tf.logging.error(
                    "Couldn't understand architecture suffix '%s' for '%s'", parts[3],
                    architecture)
                return None
            is_quantized = True
        data_url = 'http://download.tensorflow_data.org/models/mobilenet_v1_'
        data_url += version_string + '_' + size_string + '_frozen.tgz'
        bottleneck_tensor_name = 'MobilenetV1/Predictions/Reshape:0'
        bottleneck_tensor_size = 1001
        input_width = int(size_string)
        input_height = int(size_string)
        input_depth = 3
        resized_input_tensor_name = 'input:0'
        if is_quantized:
            model_base_name = 'quantized_graph.pb'
        else:
            model_base_name = 'frozen_graph.pb'
        model_dir_name = 'mobilenet_v1_' + version_string + '_' + size_string
        model_file_name = os.path.join(model_dir_name, model_base_name)
        input_mean = 127.5
        input_std = 127.5
    else:
        tf.logging.error("Couldn't understand architecture name '%s'", architecture)
        raise ValueError('Unknown architecture', architecture)

    return {
        'data_url': data_url,
        'bottleneck_tensor_name': bottleneck_tensor_name,
        'bottleneck_tensor_size': bottleneck_tensor_size,
        'input_width': input_width,
        'input_height': input_height,
        'input_depth': input_depth,
        'resized_input_tensor_name': resized_input_tensor_name,
        'model_file_name': model_file_name,
        'input_mean': input_mean,
        'input_std': input_std,
    }


def add_jpeg_decoding(input_width, input_height, input_depth, input_mean,
                      input_std):
    """Adds operations that perform JPEG decoding and resizing to the graph..
    Args:
        input_width: Desired width of the image fed into the recognizer graph.
        input_height: Desired width of the image fed into the recognizer graph.
        input_depth: Desired channels of the image fed into the recognizer graph.
        input_mean: Pixel value that should be zero in the image for the graph.
        input_std: How much to divide the pixel values by before recognition.
    Returns:
        Tensors for the node to feed JPEG data into, and the output of the
            preprocessing steps.
    """
    jpeg_data = tf.placeholder(tf.string, name='DecodeJPGInput')
    decoded_image = tf.image.decode_jpeg(jpeg_data, channels=input_depth)
    decoded_image_as_float = tf.cast(decoded_image, dtype=tf.float32)
    decoded_image_4d = tf.expand_dims(decoded_image_as_float, 0)
    resize_shape = tf.stack([input_height, input_width])
    resize_shape_as_int = tf.cast(resize_shape, dtype=tf.int32)
    resized_image = tf.image.resize_bilinear(decoded_image_4d,
                                             resize_shape_as_int)
    offset_image = tf.subtract(resized_image, input_mean)
    mul_image = tf.multiply(offset_image, 1.0 / input_std)
    return jpeg_data, mul_image


def people_training(username):

    db = get_database_con()

    training_steps = get_server_data('PEOPLE_TF_TRAINING_STEPS', db)
    try:
        how_many_training_steps = int(training_steps)
    except:
        Logger.log("error reading PEOPLE_TF_TRAINING_STEPS, using default: "+str(how_many_training_steps))

    model_size = get_server_data('PEOPLE_TF_MODEL_SIZE', db)
    image_size = get_server_data('PEOPLE_TF_MODEL_IMAGE_SIZE', db)
    try:
        model_size_string = get_graph_sizes()[int(model_size)]['value']
        image_size_string = get_image_sizes()[int(image_size)]['value']

        architecture = "mobilenet_"+model_size_string+"_"+image_size_string
    except:
        Logger.log("error reading data, using default architecture: "+str(architecture))

    #os.system("sudo python /var/www/html/python/add-event.py AR-Steuerung Trainieren der Bilder gestartet.")

    # Needed to make sure the logging output is visible.
    # See https://github.com/tensorflow/tensorflow/issues/3047
    tf.logging.set_verbosity(tf.logging.INFO)

    # Prepare necessary directories	that can be used during training
    prepare_file_system()

    # Gather information about the model architecture we'll be using.
    model_info = create_model_info(architecture)
    if not model_info:
        tf.logging.error('Did not recognize architecture flag')
        return -1

    # Set up the pre-trained graph.
    Logger.log(model_info['data_url'])
    try:
        maybe_download_and_extract(model_info['data_url'])
    except Exception as e:
            if(Logger.IS_DEBUG):
                traceback.print_exc()
    graph, bottleneck_tensor, resized_image_tensor = (
        create_model_graph(model_info))

    # Look at the folder structure, and create lists of all the images.
    image_lists = create_image_lists(image_dir, testing_percentage,
                                     validation_percentage)
    class_count = len(list(image_lists.keys()))
    if class_count == 0:
        tf.logging.error('No valid folders of images found at ' + image_dir)
        #os.system("sudo python /var/www/html/python/add-event.py AR-Steuerung Trainieren der Bilder abgeschlossen.")
        return -1
    if class_count == 1:
        tf.logging.error('Only one valid folder of images found at ' +
                         image_dir +
                         ' - multiple classes are needed for classification.')
        #os.system("sudo python /var/www/html/python/add-event.py AR-Steuerung Trainieren der Bilder abgeschlossen.")
        return -1

    # See if the command-line flags mean we're applying any distortions.
    do_distort_images = should_distort_images(
        flip_left_right, random_crop, random_scale,
        random_brightness)

    with tf.Session(graph=graph) as sess:
        # Set up the image decoding sub-graph.
        jpeg_data_tensor, decoded_image_tensor = add_jpeg_decoding(
            model_info['input_width'], model_info['input_height'],
            model_info['input_depth'], model_info['input_mean'],
            model_info['input_std'])

        if do_distort_images:
            # We will be applying distortions, so setup the operations we'll need.
            (distorted_jpeg_data_tensor,
             distorted_image_tensor) = add_input_distortions(
                flip_left_right, random_crop, random_scale,
                random_brightness, model_info['input_width'],
                model_info['input_height'], model_info['input_depth'],
                model_info['input_mean'], model_info['input_std'])
        else:
            # We'll make sure we've calculated the 'bottleneck' image summaries and
            # cached them on disk.
            cache_bottlenecks(sess, image_lists, image_dir,
                              bottleneck_dir, jpeg_data_tensor,
                              decoded_image_tensor, resized_image_tensor,
                              bottleneck_tensor, architecture)

        # Add the new layer that we'll be training.
        (train_step, cross_entropy, bottleneck_input, ground_truth_input,
         final_tensor) = add_final_training_ops(
            len(list(image_lists.keys())), final_tensor_name, bottleneck_tensor,
            model_info['bottleneck_tensor_size'])

        # Create the operations we need to evaluate the accuracy of our new layer.
        evaluation_step, prediction = add_evaluation_step(
            final_tensor, ground_truth_input)

        # Merge all the summaries and write them out to the summaries_dir
        merged = tf.summary.merge_all()
        train_writer = tf.summary.FileWriter(summaries_dir + '/train',
                                             sess.graph)

        validation_writer = tf.summary.FileWriter(
            summaries_dir + '/validation')

        # Set up all our weights to their initial default values.
        init = tf.global_variables_initializer()
        sess.run(init)

        # Run the training for as many cycles as requested on the command line.
        for i in range(how_many_training_steps):
            # Get a batch of input bottleneck values, either calculated fresh every
            # time with distortions applied, or from the cache stored on disk.
            if do_distort_images:
                (train_bottlenecks,
                 train_ground_truth) = get_random_distorted_bottlenecks(
                    sess, image_lists, train_batch_size, 'training',
                    image_dir, distorted_jpeg_data_tensor,
                    distorted_image_tensor, resized_image_tensor, bottleneck_tensor)
            else:
                (train_bottlenecks,
                 train_ground_truth, _) = get_random_cached_bottlenecks(
                    sess, image_lists, train_batch_size, 'training',
                    bottleneck_dir, image_dir, jpeg_data_tensor,
                    decoded_image_tensor, resized_image_tensor, bottleneck_tensor,
                    architecture)
            # Feed the bottlenecks and ground truth into the graph, and run a training
            # step. Capture training summaries for TensorBoard with the `merged` op.
            train_summary, _ = sess.run(
                [merged, train_step],
                feed_dict={bottleneck_input: train_bottlenecks,
                           ground_truth_input: train_ground_truth})
            train_writer.add_summary(train_summary, i)

            # Every so often, print out how well the graph is training.
            is_last_step = (i + 1 == how_many_training_steps)
            if (i % eval_step_interval) == 0 or is_last_step:
                train_accuracy, cross_entropy_value = sess.run(
                    [evaluation_step, cross_entropy],
                    feed_dict={bottleneck_input: train_bottlenecks,
                               ground_truth_input: train_ground_truth})
                tf.logging.info('%s: Step %d: Train accuracy = %.1f%%' %
                                (datetime.now(), i, train_accuracy * 100))
                tf.logging.info('%s: Step %d: Cross entropy = %f' %
                                (datetime.now(), i, cross_entropy_value))
                validation_bottlenecks, validation_ground_truth, _ = (
                    get_random_cached_bottlenecks(
                        sess, image_lists, validation_batch_size, 'validation',
                        bottleneck_dir, image_dir, jpeg_data_tensor,
                        decoded_image_tensor, resized_image_tensor, bottleneck_tensor,
                        architecture))
                # Run a validation step and capture training summaries for TensorBoard
                # with the `merged` op.
                validation_summary, validation_accuracy = sess.run(
                    [merged, evaluation_step],
                    feed_dict={bottleneck_input: validation_bottlenecks,
                               ground_truth_input: validation_ground_truth})
                validation_writer.add_summary(validation_summary, i)
                tf.logging.info('%s: Step %d: Validation accuracy = %.1f%% (N=%d)' %
                                (datetime.now(), i, validation_accuracy * 100,
                                 len(validation_bottlenecks)))

            # Store intermediate results
            intermediate_frequency = intermediate_store_frequency

            if (intermediate_frequency > 0 and (i % intermediate_frequency == 0)
                    and i > 0):
                intermediate_file_name = (intermediate_output_graphs_dir +
                                          'intermediate_' + str(i) + '.pb')
                tf.logging.info('Save intermediate result to : ' +
                                intermediate_file_name)
                save_graph_to_file(sess, graph, intermediate_file_name)

        # We've completed all our training, so run a final test evaluation on
        # some new images we haven't used before.
        test_bottlenecks, test_ground_truth, test_filenames = (
            get_random_cached_bottlenecks(
                sess, image_lists, test_batch_size, 'testing',
                bottleneck_dir, image_dir, jpeg_data_tensor,
                decoded_image_tensor, resized_image_tensor, bottleneck_tensor,
                architecture))
        test_accuracy, predictions = sess.run(
            [evaluation_step, prediction],
            feed_dict={bottleneck_input: test_bottlenecks,
                       ground_truth_input: test_ground_truth})
        tf.logging.info('Final test accuracy = %.1f%% (N=%d)' %
                        (test_accuracy * 100, len(test_bottlenecks)))

        if print_misclassified_test_images:
            tf.logging.info('=== MISCLASSIFIED TEST IMAGES ===')
            for i, test_filename in enumerate(test_filenames):
                if predictions[i] != test_ground_truth[i].argmax():
                    tf.logging.info('%70s	%s' %
                                    (test_filename,
                                     list(image_lists.keys())[predictions[i]]))

        # Write out the trained graph and labels with the weights stored as
        # constants.
        save_graph_to_file(sess, graph, output_graph)
        with gfile.FastGFile(output_labels, 'w') as f:
            f.write('\n'.join(list(image_lists.keys())) + '\n')
    #os.system("sudo python /var/www/html/python/add-event.py AR-Steuerung Trainieren der Bilder abgeschlossen.")

    send_notification_to_users(username, "Personenerkennung", "Das Training der Personenerkennung ist abgeschlossen.", db)

def get_graph_sizes():
    data = [
        {'text': "0.25 (Sehr schnell)", 'value': '0.25'},
        {'text': "0.50 (Schnell)", 'value': '0.50'},
        {'text': "0.75 (Normal)", 'value': '0.75'},
        {'text': "1.0 (Langsam)", 'value': '1.0'}
    ]

    return data

def get_image_sizes():
    data = [
        {'text': "128 (Sehr schnell)", 'value': '128'},
        {'text': "160 (Schnell)", 'value': '160'},
        {'text': "192 (Normal)", 'value': '192'},
        {'text': "224 (Langsam)", 'value': '224'}
    ]

    return data